/*----- PROTECTED REGION ID(EbsCorrector.cpp) ENABLED START -----*/
//=============================================================================
//
// file :        EbsCorrector.cpp
//
// description : C++ source for the EbsCorrector class and its commands.
//               The class is derived from Device. It represents the
//               CORBA servant object which will be accessed from the
//               network. All commands which can be executed on the
//               EbsCorrector are implemented in this file.
//
// project :     
//
// This file is part of Tango device class.
// 
// Tango is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Tango is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
// 
//
// Copyright (C): 2022
//                European Synchrotron Radiation Facility
//                BP 220, Grenoble 38043
//                France
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================


#include <EbsCorrector.h>
#include <EbsCorrectorClass.h>


/*----- PROTECTED REGION END -----*/	//	EbsCorrector.cpp

/**
 *  EbsCorrector class description:
 *    Class to wrap attributes of EbsMagnet for a single magnetic component (H/V or SQ).
 */

//================================================================
//  The following table gives the correspondence
//  between command and method names.
//
//  Command name   |  Method name
//================================================================
//  State          |  dev_state
//  Status         |  Inherited (no method)
//  SetpointCheck  |  setpoint_check
//  Reset          |  reset
//================================================================

//================================================================
//  Attributes managed are:
//================================================================
//  Strength  |  Tango::DevDouble	Scalar
//  Frozen    |  Tango::DevBoolean	Scalar
//================================================================

namespace EbsCorrector_ns
{
/*----- PROTECTED REGION ID(EbsCorrector::namespace_starting) ENABLED START -----*/

//	static initializations

/*----- PROTECTED REGION END -----*/	//	EbsCorrector::namespace_starting

//--------------------------------------------------------
/**
 *	Method      : EbsCorrector::EbsCorrector()
 *	Description : Constructors for a Tango device
 *                implementing the classEbsCorrector
 */
//--------------------------------------------------------
EbsCorrector::EbsCorrector(Tango::DeviceClass *cl, string &s)
 : TANGO_BASE_CLASS(cl, s.c_str())
{
	/*----- PROTECTED REGION ID(EbsCorrector::constructor_1) ENABLED START -----*/
  attr_ResonanceStrength_read[0] = 0;
	init_device();
	
	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::constructor_1
}
//--------------------------------------------------------
EbsCorrector::EbsCorrector(Tango::DeviceClass *cl, const char *s)
 : TANGO_BASE_CLASS(cl, s)
{
	/*----- PROTECTED REGION ID(EbsCorrector::constructor_2) ENABLED START -----*/
  attr_ResonanceStrength_read[0] = 0;
	init_device();
	
	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::constructor_2
}
//--------------------------------------------------------
EbsCorrector::EbsCorrector(Tango::DeviceClass *cl, const char *s, const char *d)
 : TANGO_BASE_CLASS(cl, s, d)
{
	/*----- PROTECTED REGION ID(EbsCorrector::constructor_3) ENABLED START -----*/
  attr_ResonanceStrength_read[0] = 0;
	init_device();
	
	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::constructor_3
}

//--------------------------------------------------------
/**
 *	Method      : EbsCorrector::delete_device()
 *	Description : will be called at device destruction or at init command
 */
//--------------------------------------------------------
void EbsCorrector::delete_device()
{
	DEBUG_STREAM << "EbsCorrector::delete_device() " << device_name << endl;
	/*----- PROTECTED REGION ID(EbsCorrector::delete_device) ENABLED START -----*/
  delete mDS;
	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::delete_device
	delete[] attr_Strength_read;
	delete[] attr_Frozen_read;
}

//--------------------------------------------------------
/**
 *	Method      : EbsCorrector::init_device()
 *	Description : will be called at device initialization.
 */
//--------------------------------------------------------
void EbsCorrector::init_device()
{
	DEBUG_STREAM << "EbsCorrector::init_device() create device " << device_name << endl;
	/*----- PROTECTED REGION ID(EbsCorrector::init_device_before) ENABLED START -----*/
	
	//	Initialization before get_device_property() call
  mDS = nullptr;
	
	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::init_device_before
	

	//	Get the device properties from database
	get_device_property();
	
	attr_Strength_read = new Tango::DevDouble[1];
	attr_Frozen_read = new Tango::DevBoolean[1];
	/*----- PROTECTED REGION ID(EbsCorrector::init_device) ENABLED START -----*/

  if( magnetDevice.empty() ) {
    cerr << "Fatal Error: MagnetDevice property not defined for " << get_name() << endl;
    exit(0);
  }
  if( type.empty() ) {
    cerr << "Fatal Error: Type property not defined for " << get_name() << endl;
    exit(0);
  }

  isSQ = false;
  if( type=="H" ) {
    pCurrentIdx = 0;
  } else if (type=="V") {
    pCurrentIdx = 1;
  } else if (type=="SQ") {
    pCurrentIdx = 2;
    isSQ = true;
  } else {
    cerr << "Fatal Error: Type property wrongly defined for " << get_name() << " H,V, or SQ expected" << endl;
    exit(0);
  }

  try {
    mDS = new Tango::DeviceProxy(magnetDevice);
    mDS->set_source(Tango::DEV);
    mDS->set_timeout_millis(3900); // MagnetType and CorrectorType have a 4 sec timeout, EbsMagnet 3800
  } catch(Tango::DevFailed &e) {
    cerr << "Fatal: Cannot import " << magnetDevice << endl;
    cerr << e.errors[0].desc << endl;
    exit(0);
  }

  // FOFB H & V steering pseudo currents
  hasPseudoCurrent = magnetDevice.find("-sh")!=std::string::npos;

  // Set units
  set_att_unit("Strength",(isSQ)?"m-1":"rad");

  strengthAtt = &get_device_attr()->get_w_attr_by_name("Strength");
  attr_Strength_read[0] = 0;
  attr_CorrectionStrength_read[0] = 0;

	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::init_device
}

//--------------------------------------------------------
/**
 *	Method      : EbsCorrector::get_device_property()
 *	Description : Read database to initialize property data members.
 */
//--------------------------------------------------------
void EbsCorrector::get_device_property()
{
	/*----- PROTECTED REGION ID(EbsCorrector::get_device_property_before) ENABLED START -----*/
	
	//	Initialize property data members
	
	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::get_device_property_before


	//	Read device properties from database.
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("MagnetDevice"));
	dev_prop.push_back(Tango::DbDatum("Type"));

	//	is there at least one property to be read ?
	if (dev_prop.size()>0)
	{
		//	Call database and extract values
		if (Tango::Util::instance()->_UseDb==true)
			get_db_device()->get_property(dev_prop);
	
		//	get instance on EbsCorrectorClass to get class property
		Tango::DbDatum	def_prop, cl_prop;
		EbsCorrectorClass	*ds_class =
			(static_cast<EbsCorrectorClass *>(get_device_class()));
		int	i = -1;

		//	Try to initialize MagnetDevice from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  magnetDevice;
		else {
			//	Try to initialize MagnetDevice from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  magnetDevice;
		}
		//	And try to extract MagnetDevice value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  magnetDevice;

		//	Try to initialize Type from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  type;
		else {
			//	Try to initialize Type from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  type;
		}
		//	And try to extract Type value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  type;

	}

	/*----- PROTECTED REGION ID(EbsCorrector::get_device_property_after) ENABLED START -----*/
	
	//	Check device property data members init
	
	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::get_device_property_after
}

//--------------------------------------------------------
/**
 *	Method      : EbsCorrector::always_executed_hook()
 *	Description : method always executed before any command is executed
 */
//--------------------------------------------------------
void EbsCorrector::always_executed_hook()
{
	DEBUG_STREAM << "EbsCorrector::always_executed_hook()  " << device_name << endl;
	/*----- PROTECTED REGION ID(EbsCorrector::always_executed_hook) ENABLED START -----*/
	
	//	code always executed before all requests
	
	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::always_executed_hook
}

//--------------------------------------------------------
/**
 *	Method      : EbsCorrector::read_attr_hardware()
 *	Description : Hardware acquisition for attributes
 */
//--------------------------------------------------------
void EbsCorrector::read_attr_hardware(TANGO_UNUSED(vector<long> &attr_list))
{
	DEBUG_STREAM << "EbsCorrector::read_attr_hardware(vector<long> &attr_list) entering... " << endl;
	/*----- PROTECTED REGION ID(EbsCorrector::read_attr_hardware) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::read_attr_hardware
}
//--------------------------------------------------------
/**
 *	Method      : EbsCorrector::write_attr_hardware()
 *	Description : Hardware writing for attributes
 */
//--------------------------------------------------------
void EbsCorrector::write_attr_hardware(TANGO_UNUSED(vector<long> &attr_list))
{
	DEBUG_STREAM << "EbsCorrector::write_attr_hardware(vector<long> &attr_list) entering... " << endl;
	/*----- PROTECTED REGION ID(EbsCorrector::write_attr_hardware) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::write_attr_hardware
}

//--------------------------------------------------------
/**
 *	Read attribute Strength related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void EbsCorrector::read_Strength(Tango::Attribute &attr)
{
	DEBUG_STREAM << "EbsCorrector::read_Strength(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(EbsCorrector::read_Strength) ENABLED START -----*/
  read_attribute<Tango::DevDouble>("Strength",attr,attr_Strength_read);
  update_correction_strength();
	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::read_Strength
}
//--------------------------------------------------------
/**
 *	Write attribute Strength related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void EbsCorrector::write_Strength(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "EbsCorrector::write_Strength(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(EbsCorrector::write_Strength) ENABLED START -----*/
  if(!is_setpoint_valid(w_val)) {
    attr.rollback();
    return;
  }
  Tango::DeviceAttribute da(("Strength_" + type).c_str(),w_val);
  mDS->write_attribute(da);
  update_correction_strength();
	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::write_Strength
}
//--------------------------------------------------------
/**
 *	Read attribute Frozen related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void EbsCorrector::read_Frozen(Tango::Attribute &attr)
{
	DEBUG_STREAM << "EbsCorrector::read_Frozen(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(EbsCorrector::read_Frozen) ENABLED START -----*/
  read_attribute<Tango::DevBoolean>("Frozen",attr,attr_Frozen_read);
	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::read_Frozen
}
//--------------------------------------------------------
/**
 *	Write attribute Frozen related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void EbsCorrector::write_Frozen(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "EbsCorrector::write_Frozen(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevBoolean	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(EbsCorrector::write_Frozen) ENABLED START -----*/
  Tango::DeviceAttribute da(("Frozen_" + type).c_str(),w_val);
  mDS->write_attribute(da);
  /*----- PROTECTED REGION END -----*/	//	EbsCorrector::write_Frozen
}

//--------------------------------------------------------
/**
 *	Read attribute CorrectionStrength related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void EbsCorrector::read_CorrectionStrength(Tango::Attribute &attr)
{
	DEBUG_STREAM << "EbsCorrector::read_CorrectionStrength(Tango::Attribute &attr) entering... " << endl;
	Tango::DevDouble	*att_value = get_CorrectionStrength_data_ptr(attr.get_name());
	/*----- PROTECTED REGION ID(EbsCorrector::read_CorrectionStrength) ENABLED START -----*/
  // CorrectionStrength is a local attribute to SQ corrector
  attr.set_value(attr_CorrectionStrength_read);
	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::read_CorrectionStrength
}
//--------------------------------------------------------
/**
 *	Write attribute CorrectionStrength related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void EbsCorrector::write_CorrectionStrength(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "EbsCorrector::write_CorrectionStrength(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(EbsCorrector::write_CorrectionStrength) ENABLED START -----*/
  if(!is_setpoint_valid(w_val)) {
    attr.rollback();
    return;
  }

  Tango::DevDouble newStrength;
  resonanceAtt->get_write_value(newStrength);
  newStrength += w_val;
  Tango::DeviceAttribute da(("Strength_" + type).c_str(),newStrength);
  mDS->write_attribute(da);
  update_correction_strength();

	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::write_CorrectionStrength
}
//--------------------------------------------------------
/**
 *	Read attribute ResonanceStrength related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void EbsCorrector::read_ResonanceStrength(Tango::Attribute &attr)
{
	DEBUG_STREAM << "EbsCorrector::read_ResonanceStrength(Tango::Attribute &attr) entering... " << endl;
	Tango::DevDouble	*att_value = get_ResonanceStrength_data_ptr(attr.get_name());
	/*----- PROTECTED REGION ID(EbsCorrector::read_ResonanceStrength) ENABLED START -----*/
  // ResonanceStrength is a local attribute to SQ corrector
	attr.set_value(attr_ResonanceStrength_read);
	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::read_ResonanceStrength
}
//--------------------------------------------------------
/**
 *	Write attribute ResonanceStrength related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void EbsCorrector::write_ResonanceStrength(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "EbsCorrector::write_ResonanceStrength(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(EbsCorrector::write_ResonanceStrength) ENABLED START -----*/

  if(!is_setpoint_valid(w_val)) {
    attr.rollback();
    return;
  }

  Tango::DevDouble newStrength;
  correctionAtt->get_write_value(newStrength);
  newStrength += w_val;
  Tango::DeviceAttribute da(("Strength_" + type).c_str(),newStrength);
  mDS->write_attribute(da);
  update_correction_strength();

  /*----- PROTECTED REGION END -----*/	//	EbsCorrector::write_ResonanceStrength
}
//--------------------------------------------------------
/**
 *	Read attribute Current related method
 *	Description: Steering pseudo current
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void EbsCorrector::read_Current(Tango::Attribute &attr)
{
	DEBUG_STREAM << "EbsCorrector::read_Current(Tango::Attribute &attr) entering... " << endl;
	Tango::DevDouble	*att_value = get_Current_data_ptr(attr.get_name());
	/*----- PROTECTED REGION ID(EbsCorrector::read_Current) ENABLED START -----*/

  if(!hasPseudoCurrent) {
    attr.set_quality(Tango::ATTR_INVALID);
    return;
  }

  Tango::DeviceData argin;
  argin << (Tango::DevShort)pCurrentIdx;
  vector<double> argout;
  mDS->command_inout("GetPSCurrent",argin) >> argout;
  att_value[0] = argout[0];
	attr.set_value(att_value);
  (static_cast<Tango::WAttribute&>(attr)).set_write_value(argout[1]);

	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::read_Current
}
//--------------------------------------------------------
/**
 *	Write attribute Current related method
 *	Description: Steering pseudo current
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void EbsCorrector::write_Current(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "EbsCorrector::write_Current(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(EbsCorrector::write_Current) ENABLED START -----*/

  if(!hasPseudoCurrent || !is_setpoint_valid(w_val)) {
    attr.rollback();
    return;
  }
  vector<double> argin;
  argin.emplace_back(pCurrentIdx);
  argin.emplace_back(w_val);
  Tango::DeviceData dd;
  dd << argin;
  mDS->command_inout("WritePSCurrent",dd);
  update_correction_strength();

  /*----- PROTECTED REGION END -----*/	//	EbsCorrector::write_Current
}
//--------------------------------------------------------
/**
 *	Read attribute SetCurrentsAverage related method
 *	Description: Average AC pseudo current
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void EbsCorrector::read_SetCurrentsAverage(Tango::Attribute &attr)
{
	DEBUG_STREAM << "EbsCorrector::read_SetCurrentsAverage(Tango::Attribute &attr) entering... " << endl;
	Tango::DevDouble	*att_value = get_SetCurrentsAverage_data_ptr(attr.get_name());
	/*----- PROTECTED REGION ID(EbsCorrector::read_SetCurrentsAverage) ENABLED START -----*/

  Tango::DeviceData argin;
  argin << (Tango::DevShort)pCurrentIdx;
  mDS->command_inout("GetACPSCurrent",argin) >> att_value[0];
	attr.set_value(att_value);
	
	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::read_SetCurrentsAverage
}
//--------------------------------------------------------
/**
 *	Method      : EbsCorrector::add_dynamic_attributes()
 *	Description : Create the dynamic attributes if any
 *                for specified device.
 */
//--------------------------------------------------------
void EbsCorrector::add_dynamic_attributes()
{
	//	Example to add dynamic attribute:
	//	Copy inside the following protected area to create instance(s) at startup.
	//	add_CorrectionStrength_dynamic_attribute("MyCorrectionStrengthAttribute");
	//	add_ResonanceStrength_dynamic_attribute("MyResonanceStrengthAttribute");
	//	add_Current_dynamic_attribute("MyCurrentAttribute");
	//	add_SetCurrentsAverage_dynamic_attribute("MySetCurrentsAverageAttribute");
	
	/*----- PROTECTED REGION ID(EbsCorrector::add_dynamic_attributes) ENABLED START -----*/

  if(isSQ) {
    add_CorrectionStrength_dynamic_attribute("CorrectionStrength");
    add_ResonanceStrength_dynamic_attribute("ResonanceStrength");
    correctionAtt = &get_device_attr()->get_w_attr_by_name("CorrectionStrength");
    resonanceAtt = &get_device_attr()->get_w_attr_by_name("ResonanceStrength");;
  }

  if(hasPseudoCurrent) {
    add_Current_dynamic_attribute("Current");
    add_SetCurrentsAverage_dynamic_attribute("SetCurrentsAverage");
    currentAtt = &get_device_attr()->get_w_attr_by_name("Current");
  }

	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::add_dynamic_attributes
}

//--------------------------------------------------------
/**
 *	Command State related method
 *	Description: This command gets the device state (stored in its device_state data member) and returns it to the caller.
 *
 *	@returns Device state
 */
//--------------------------------------------------------
Tango::DevState EbsCorrector::dev_state()
{
	DEBUG_STREAM << "EbsCorrector::State()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(EbsCorrector::dev_state) ENABLED START -----*/

	Tango::DevState	argout;
  mDS->read_attribute("CorrectorState") >> argout;
  if(attr_Frozen_read[0] && argout!=Tango::FAULT && argout!=Tango::UNKNOWN) {
    argout = Tango::DISABLE;
    set_status("The corrector is frozen");
  } else {
    string magStatus;
    mDS->read_attribute("Status") >> magStatus;
    set_status(magStatus);
  }

	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::dev_state
	set_state(argout);    // Give the state to Tango.
	if (argout!=Tango::ALARM)
		Tango::DeviceImpl::dev_state();
	return get_state();  // Return it after Tango management.
}
//--------------------------------------------------------
/**
 *	Command SetpointCheck related method
 *	Description: True if it`s possible for the PS to generate the currents required
 *               to deliver the strength given as argument
 *
 *	@param argin 
 *	@returns 
 */
//--------------------------------------------------------
Tango::DevBoolean EbsCorrector::setpoint_check(Tango::DevDouble argin)
{
	Tango::DevBoolean argout;
	DEBUG_STREAM << "EbsCorrector::SetpointCheck()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(EbsCorrector::setpoint_check) ENABLED START -----*/

  if(isnan(argin) || attr_Frozen_read[0])
    return true;

  Tango::DeviceData dd;
  dd << argin;
  string cmdName = "SetpointCheck_" + type;
  mDS->command_inout(cmdName,dd) >> argout;

	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::setpoint_check
	return argout;
}
//--------------------------------------------------------
/**
 *	Command Reset related method
 *	Description: Reset the corrector
 *
 */
//--------------------------------------------------------
void EbsCorrector::reset()
{
	DEBUG_STREAM << "EbsCorrector::Reset()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(EbsCorrector::reset) ENABLED START -----*/
	mDS->command_inout("Reset");
	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::reset
}
//--------------------------------------------------------
/**
 *	Command ResetResonanceStrength related method
 *	Description: Reset ResonanceStrength to 0
 *
 */
//--------------------------------------------------------
void EbsCorrector::reset_resonance_strength(Tango::Command &command)
{
	DEBUG_STREAM << "EbsCorrector::" << command.get_name() << "  - " << device_name << endl;
	/*----- PROTECTED REGION ID(EbsCorrector::reset_resonance_strength) ENABLED START -----*/
  if( attr_ResonanceStrength_read[0]!=0.0 ) {
    attr_ResonanceStrength_read[0] = 0;
    resonanceAtt->set_write_value((Tango::DevDouble) 0);
    update_correction_strength();
    save_mem_to_db(get_name(),"ResonanceStrength", "0");
  }
	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::reset_resonance_strength
}
//--------------------------------------------------------
/**
 *	Method      : EbsCorrector::add_dynamic_commands()
 *	Description : Create the dynamic commands if any
 *                for specified device.
 */
//--------------------------------------------------------
void EbsCorrector::add_dynamic_commands()
{
	//	Example to add dynamic command:
	//	Copy inside the folowing protected area to instanciate at startup.
	//	add_ResetResonanceStrength_dynamic_command("MyResetResonanceStrengthCommand", true);
	
	/*----- PROTECTED REGION ID(EbsCorrector::add_dynamic_commands) ENABLED START -----*/

  if(isSQ)
    add_ResetResonanceStrength_dynamic_command("ResetResonanceStrength", true);

  /*----- PROTECTED REGION END -----*/	//	EbsCorrector::add_dynamic_commands
}

/*----- PROTECTED REGION ID(EbsCorrector::namespace_ending) ENABLED START -----*/

//	Additional Methods

void EbsCorrector::set_att_unit(const string &_att_name,const string &_unit) {
  Tango::MultiAttrProp<Tango::DevDouble> props;
  Tango::Attribute &att = get_device_attr()->get_attr_by_name(_att_name.c_str());
  att.get_properties(props);
  props.unit = _unit;
  att.set_properties(props);
}

template<typename inType>
void EbsCorrector::read_attribute(const string &name,Tango::Attribute& attr,inType* readValue) {

  string attName;
  attName.reserve(128);
  attName.append(name);
  attName.append("_");
  attName.append(type);
  vector<inType> values;
  mDS->read_attribute(attName) >> values;
  *readValue = values[0];
  attr.set_value(readValue);
  // Update setpoint
  if(attr.get_writable() == Tango::READ_WRITE) {
    inType set = values[1];
    (static_cast<Tango::WAttribute&>(attr)).set_write_value(set);
  }

}

void EbsCorrector::update_correction_strength() {

  if(type=="SQ") {
    Tango::DevDouble strength;
    Tango::DevDouble rsStrength;
    strengthAtt->get_write_value(strength);
    resonanceAtt->get_write_value(rsStrength);
    correctionAtt->set_write_value(strength - rsStrength);
    attr_CorrectionStrength_read[0] = attr_Strength_read[0] - attr_ResonanceStrength_read[0];
    attr_ResonanceStrength_read[0] = rsStrength;
  }

}

bool EbsCorrector::is_setpoint_valid(double value) {
  if(isnan(value))
    return false;
  if(attr_Frozen_read[0])
    return false;
  return true;
}

void EbsCorrector::save_mem_to_db(const string& devname,const string& attname,const string& value) {

  // Save to DB
  Tango::Database *db = Tango::Util::instance()->get_database();
  Tango::DbDatum reso(attname);
  reso << 1;
  Tango::DbDatum val("__value");
  val << value.c_str();
  Tango::DbData db_data = {reso, val};
  db->put_device_attribute_property(devname, db_data);

}

/*----- PROTECTED REGION END -----*/	//	EbsCorrector::namespace_ending
} //	namespace
