/*----- PROTECTED REGION ID(EbsCorrector.h) ENABLED START -----*/
//=============================================================================
//
// file :        EbsCorrector.h
//
// description : Include file for the EbsCorrector class
//
// project :     
//
// This file is part of Tango device class.
// 
// Tango is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Tango is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
// 
//
// Copyright (C): 2022
//                European Synchrotron Radiation Facility
//                BP 220, Grenoble 38043
//                France
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================


#ifndef EbsCorrector_H
#define EbsCorrector_H

#include <tango.h>

/*----- PROTECTED REGION END -----*/	//	EbsCorrector.h

/**
 *  EbsCorrector class description:
 *    Class to wrap attributes of EbsMagnet for a single magnetic component (H/V or SQ).
 */

namespace EbsCorrector_ns
{
/*----- PROTECTED REGION ID(EbsCorrector::Additional Class Declarations) ENABLED START -----*/

//	Additional Class Declarations

/*----- PROTECTED REGION END -----*/	//	EbsCorrector::Additional Class Declarations

class EbsCorrector : public TANGO_BASE_CLASS
{

/*----- PROTECTED REGION ID(EbsCorrector::Data Members) ENABLED START -----*/

//	Add your own data members

private:
  Tango::DeviceProxy *mDS;

  void set_att_unit(const string &_att_name,const string &_unit);

  template<typename inType>
  void read_attribute(const string &name,Tango::Attribute& attr,inType* readValue);
  void update_correction_strength();
  bool is_setpoint_valid(double value);

  Tango::WAttribute *strengthAtt;
  Tango::WAttribute *correctionAtt;
  Tango::WAttribute *resonanceAtt;
  Tango::WAttribute *currentAtt;

  bool isSQ;             // Is Skew Quand flag
  bool hasPseudoCurrent; // Has pseudo current flag (SH H & V), used by FOFB
  int  pCurrentIdx;      // pseudo current index

  Tango::DevDouble attr_CorrectionStrength_read[1];
  Tango::DevDouble attr_ResonanceStrength_read[1];

  void save_mem_to_db(const string& devname,const string& attname,const string& value);

/*----- PROTECTED REGION END -----*/	//	EbsCorrector::Data Members

//	Device property data members
public:
	//	MagnetDevice:	Name of the Magnet device
	string	magnetDevice;
	//	Type:	Magnetic component to wrap (H,V or SQ)
	string	type;

//	Attribute data members
public:
	Tango::DevDouble	*attr_Strength_read;
	Tango::DevBoolean	*attr_Frozen_read;

//	Constructors and destructors
public:
	/**
	 * Constructs a newly device object.
	 *
	 *	@param cl	Class.
	 *	@param s 	Device Name
	 */
	EbsCorrector(Tango::DeviceClass *cl,string &s);
	/**
	 * Constructs a newly device object.
	 *
	 *	@param cl	Class.
	 *	@param s 	Device Name
	 */
	EbsCorrector(Tango::DeviceClass *cl,const char *s);
	/**
	 * Constructs a newly device object.
	 *
	 *	@param cl	Class.
	 *	@param s 	Device name
	 *	@param d	Device description.
	 */
	EbsCorrector(Tango::DeviceClass *cl,const char *s,const char *d);
	/**
	 * The device object destructor.
	 */
	~EbsCorrector() {delete_device();};


//	Miscellaneous methods
public:
	/*
	 *	will be called at device destruction or at init command.
	 */
	void delete_device();
	/*
	 *	Initialize the device
	 */
	virtual void init_device();
	/*
	 *	Read the device properties from database
	 */
	void get_device_property();
	/*
	 *	Always executed method before execution command method.
	 */
	virtual void always_executed_hook();


//	Attribute methods
public:
	//--------------------------------------------------------
	/*
	 *	Method      : EbsCorrector::read_attr_hardware()
	 *	Description : Hardware acquisition for attributes.
	 */
	//--------------------------------------------------------
	virtual void read_attr_hardware(vector<long> &attr_list);
	//--------------------------------------------------------
	/*
	 *	Method      : EbsCorrector::write_attr_hardware()
	 *	Description : Hardware writing for attributes.
	 */
	//--------------------------------------------------------
	virtual void write_attr_hardware(vector<long> &attr_list);

/**
 *	Attribute Strength related methods
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
	virtual void read_Strength(Tango::Attribute &attr);
	virtual void write_Strength(Tango::WAttribute &attr);
	virtual bool is_Strength_allowed(Tango::AttReqType type);
/**
 *	Attribute Frozen related methods
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
	virtual void read_Frozen(Tango::Attribute &attr);
	virtual void write_Frozen(Tango::WAttribute &attr);
	virtual bool is_Frozen_allowed(Tango::AttReqType type);

//	Dynamic attribute methods
public:

	/**
	 *	Attribute CorrectionStrength related methods
	 *	Description: 
	 *
	 *	Data type:	Tango::DevDouble
	 *	Attr type:	Scalar
	 */
	virtual void read_CorrectionStrength(Tango::Attribute &attr);
	virtual void write_CorrectionStrength(Tango::WAttribute &attr);
	virtual bool is_CorrectionStrength_allowed(Tango::AttReqType type);
	void add_CorrectionStrength_dynamic_attribute(string attname);
	void remove_CorrectionStrength_dynamic_attribute(string attname);
	Tango::DevDouble *get_CorrectionStrength_data_ptr(string &name);
	map<string,Tango::DevDouble>	   CorrectionStrength_data;

	/**
	 *	Attribute ResonanceStrength related methods
	 *	Description: 
	 *
	 *	Data type:	Tango::DevDouble
	 *	Attr type:	Scalar
	 */
	virtual void read_ResonanceStrength(Tango::Attribute &attr);
	virtual void write_ResonanceStrength(Tango::WAttribute &attr);
	virtual bool is_ResonanceStrength_allowed(Tango::AttReqType type);
	void add_ResonanceStrength_dynamic_attribute(string attname);
	void remove_ResonanceStrength_dynamic_attribute(string attname);
	Tango::DevDouble *get_ResonanceStrength_data_ptr(string &name);
	map<string,Tango::DevDouble>	   ResonanceStrength_data;

	/**
	 *	Attribute Current related methods
	 *	Description: Steering pseudo current
	 *
	 *	Data type:	Tango::DevDouble
	 *	Attr type:	Scalar
	 */
	virtual void read_Current(Tango::Attribute &attr);
	virtual void write_Current(Tango::WAttribute &attr);
	virtual bool is_Current_allowed(Tango::AttReqType type);
	void add_Current_dynamic_attribute(string attname);
	void remove_Current_dynamic_attribute(string attname);
	Tango::DevDouble *get_Current_data_ptr(string &name);
	map<string,Tango::DevDouble>	   Current_data;

	/**
	 *	Attribute SetCurrentsAverage related methods
	 *	Description: Average AC pseudo current
	 *
	 *	Data type:	Tango::DevDouble
	 *	Attr type:	Scalar
	 */
	virtual void read_SetCurrentsAverage(Tango::Attribute &attr);
	virtual bool is_SetCurrentsAverage_allowed(Tango::AttReqType type);
	void add_SetCurrentsAverage_dynamic_attribute(string attname);
	void remove_SetCurrentsAverage_dynamic_attribute(string attname);
	Tango::DevDouble *get_SetCurrentsAverage_data_ptr(string &name);
	map<string,Tango::DevDouble>	   SetCurrentsAverage_data;

	//--------------------------------------------------------
	/**
	 *	Method      : EbsCorrector::add_dynamic_attributes()
	 *	Description : Add dynamic attributes if any.
	 */
	//--------------------------------------------------------
	void add_dynamic_attributes();




//	Command related methods
public:
	/**
	 *	Command State related method
	 *	Description: This command gets the device state (stored in its device_state data member) and returns it to the caller.
	 *
	 *	@returns Device state
	 */
	virtual Tango::DevState dev_state();
	/**
	 *	Command SetpointCheck related method
	 *	Description: True if it`s possible for the PS to generate the currents required
	 *               to deliver the strength given as argument
	 *
	 *	@param argin 
	 *	@returns 
	 */
	virtual Tango::DevBoolean setpoint_check(Tango::DevDouble argin);
	virtual bool is_SetpointCheck_allowed(const CORBA::Any &any);
	/**
	 *	Command Reset related method
	 *	Description: Reset the corrector
	 *
	 */
	virtual void reset();
	virtual bool is_Reset_allowed(const CORBA::Any &any);

//	Dynamic commands methods
public:
	/**
	 *	Command ResetResonanceStrength related method
	 *	Description: Reset ResonanceStrength to 0
	 *
	 */
	virtual void reset_resonance_strength(Tango::Command &command);
	virtual bool is_ResetResonanceStrength_allowed(const CORBA::Any &any);
	void add_ResetResonanceStrength_dynamic_command(string cmdname, bool device);
	void remove_ResetResonanceStrength_dynamic_command(string cmdname);

	//--------------------------------------------------------
	/**
	 *	Method      : EbsCorrector::add_dynamic_commands()
	 *	Description : Add dynamic commands if any.
	 */
	//--------------------------------------------------------
	void add_dynamic_commands();

/*----- PROTECTED REGION ID(EbsCorrector::Additional Method prototypes) ENABLED START -----*/

//	Additional Method prototypes

/*----- PROTECTED REGION END -----*/	//	EbsCorrector::Additional Method prototypes
};

/*----- PROTECTED REGION ID(EbsCorrector::Additional Classes Definitions) ENABLED START -----*/

//	Additional Classes Definitions

/*----- PROTECTED REGION END -----*/	//	EbsCorrector::Additional Classes Definitions

}	//	End of namespace

#endif   //	EbsCorrector_H
