/*----- PROTECTED REGION ID(EbsCorrectorClass.cpp) ENABLED START -----*/
//=============================================================================
//
// file :        EbsCorrectorClass.cpp
//
// description : C++ source for the EbsCorrectorClass.
//               A singleton class derived from DeviceClass.
//               It implements the command and attribute list
//               and all properties and methods required
//               by the EbsCorrector once per process.
//
// project :     
//
// This file is part of Tango device class.
// 
// Tango is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Tango is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
// 
//
// Copyright (C): 2022
//                European Synchrotron Radiation Facility
//                BP 220, Grenoble 38043
//                France
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================


#include <EbsCorrectorClass.h>

/*----- PROTECTED REGION END -----*/	//	EbsCorrectorClass.cpp

//-------------------------------------------------------------------
/**
 *	Create EbsCorrectorClass singleton and
 *	return it in a C function for Python usage
 */
//-------------------------------------------------------------------
extern "C" {
#ifdef _TG_WINDOWS_

__declspec(dllexport)

#endif

	Tango::DeviceClass *_create_EbsCorrector_class(const char *name) {
		return EbsCorrector_ns::EbsCorrectorClass::init(name);
	}
}

namespace EbsCorrector_ns
{
//===================================================================
//	Initialize pointer for singleton pattern
//===================================================================
EbsCorrectorClass *EbsCorrectorClass::_instance = NULL;

//--------------------------------------------------------
/**
 * method : 		EbsCorrectorClass::EbsCorrectorClass(string &s)
 * description : 	constructor for the EbsCorrectorClass
 *
 * @param s	The class name
 */
//--------------------------------------------------------
EbsCorrectorClass::EbsCorrectorClass(string &s):Tango::DeviceClass(s)
{
	cout2 << "Entering EbsCorrectorClass constructor" << endl;
	set_default_property();
	write_class_property();

	/*----- PROTECTED REGION ID(EbsCorrectorClass::constructor) ENABLED START -----*/
	
	/*----- PROTECTED REGION END -----*/	//	EbsCorrectorClass::constructor

	cout2 << "Leaving EbsCorrectorClass constructor" << endl;
}

//--------------------------------------------------------
/**
 * method : 		EbsCorrectorClass::~EbsCorrectorClass()
 * description : 	destructor for the EbsCorrectorClass
 */
//--------------------------------------------------------
EbsCorrectorClass::~EbsCorrectorClass()
{
	/*----- PROTECTED REGION ID(EbsCorrectorClass::destructor) ENABLED START -----*/
	
	/*----- PROTECTED REGION END -----*/	//	EbsCorrectorClass::destructor

	_instance = NULL;
}


//--------------------------------------------------------
/**
 * method : 		EbsCorrectorClass::init
 * description : 	Create the object if not already done.
 *                  Otherwise, just return a pointer to the object
 *
 * @param	name	The class name
 */
//--------------------------------------------------------
EbsCorrectorClass *EbsCorrectorClass::init(const char *name)
{
	if (_instance == NULL)
	{
		try
		{
			string s(name);
			_instance = new EbsCorrectorClass(s);
		}
		catch (bad_alloc &)
		{
			throw;
		}
	}
	return _instance;
}

//--------------------------------------------------------
/**
 * method : 		EbsCorrectorClass::instance
 * description : 	Check if object already created,
 *                  and return a pointer to the object
 */
//--------------------------------------------------------
EbsCorrectorClass *EbsCorrectorClass::instance()
{
	if (_instance == NULL)
	{
		cerr << "Class is not initialised !!" << endl;
		exit(-1);
	}
	return _instance;
}



//===================================================================
//	Command execution method calls
//===================================================================
//--------------------------------------------------------
/**
 * method : 		SetpointCheckClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *SetpointCheckClass::execute(Tango::DeviceImpl *device, const CORBA::Any &in_any)
{
	cout2 << "SetpointCheckClass::execute(): arrived" << endl;
	Tango::DevDouble argin;
	extract(in_any, argin);
	return insert((static_cast<EbsCorrector *>(device))->setpoint_check(argin));
}

//--------------------------------------------------------
/**
 * method : 		ResetClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *ResetClass::execute(Tango::DeviceImpl *device, TANGO_UNUSED(const CORBA::Any &in_any))
{
	cout2 << "ResetClass::execute(): arrived" << endl;
	((static_cast<EbsCorrector *>(device))->reset());
	return new CORBA::Any();
}

//--------------------------------------------------------
/**
 * method : 		ResetResonanceStrengthClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *ResetResonanceStrengthClass::execute(Tango::DeviceImpl *device, TANGO_UNUSED(const CORBA::Any &in_any))
{
	cout2 << "ResetResonanceStrengthClass::execute(): arrived" << endl;
	((static_cast<EbsCorrector *>(device))->reset_resonance_strength(*this));
	return new CORBA::Any();
}


//===================================================================
//	Properties management
//===================================================================
//--------------------------------------------------------
/**
 *	Method      : EbsCorrectorClass::get_class_property()
 *	Description : Get the class property for specified name.
 */
//--------------------------------------------------------
Tango::DbDatum EbsCorrectorClass::get_class_property(string &prop_name)
{
	for (unsigned int i=0 ; i<cl_prop.size() ; i++)
		if (cl_prop[i].name == prop_name)
			return cl_prop[i];
	//	if not found, returns  an empty DbDatum
	return Tango::DbDatum(prop_name);
}

//--------------------------------------------------------
/**
 *	Method      : EbsCorrectorClass::get_default_device_property()
 *	Description : Return the default value for device property.
 */
//--------------------------------------------------------
Tango::DbDatum EbsCorrectorClass::get_default_device_property(string &prop_name)
{
	for (unsigned int i=0 ; i<dev_def_prop.size() ; i++)
		if (dev_def_prop[i].name == prop_name)
			return dev_def_prop[i];
	//	if not found, return  an empty DbDatum
	return Tango::DbDatum(prop_name);
}

//--------------------------------------------------------
/**
 *	Method      : EbsCorrectorClass::get_default_class_property()
 *	Description : Return the default value for class property.
 */
//--------------------------------------------------------
Tango::DbDatum EbsCorrectorClass::get_default_class_property(string &prop_name)
{
	for (unsigned int i=0 ; i<cl_def_prop.size() ; i++)
		if (cl_def_prop[i].name == prop_name)
			return cl_def_prop[i];
	//	if not found, return  an empty DbDatum
	return Tango::DbDatum(prop_name);
}


//--------------------------------------------------------
/**
 *	Method      : EbsCorrectorClass::set_default_property()
 *	Description : Set default property (class and device) for wizard.
 *                For each property, add to wizard property name and description.
 *                If default value has been set, add it to wizard property and
 *                store it in a DbDatum.
 */
//--------------------------------------------------------
void EbsCorrectorClass::set_default_property()
{
	string	prop_name;
	string	prop_desc;
	string	prop_def;
	vector<string>	vect_data;

	//	Set Default Class Properties

	//	Set Default device Properties
	prop_name = "MagnetDevice";
	prop_desc = "Name of the Magnet device";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "Type";
	prop_desc = "Magnetic component to wrap (H,V or SQ)";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
}

//--------------------------------------------------------
/**
 *	Method      : EbsCorrectorClass::write_class_property()
 *	Description : Set class description fields as property in database
 */
//--------------------------------------------------------
void EbsCorrectorClass::write_class_property()
{
	//	First time, check if database used
	if (Tango::Util::_UseDb == false)
		return;

	Tango::DbData	data;
	string	classname = get_name();
	string	header;
	string::size_type	start, end;

	//	Put title
	Tango::DbDatum	title("ProjectTitle");
	string	str_title("");
	title << str_title;
	data.push_back(title);

	//	Put Description
	Tango::DbDatum	description("Description");
	vector<string>	str_desc;
	str_desc.push_back("Class to wrap attributes of EbsMagnet for a single magnetic component (H/V or SQ).");
	description << str_desc;
	data.push_back(description);

	//  Put inheritance
	Tango::DbDatum	inher_datum("InheritedFrom");
	vector<string> inheritance;
	inheritance.push_back("TANGO_BASE_CLASS");
	inher_datum << inheritance;
	data.push_back(inher_datum);

	//	Call database and and values
	get_db_class()->put_property(data);
}

//===================================================================
//	Factory methods
//===================================================================

//--------------------------------------------------------
/**
 *	Method      : EbsCorrectorClass::device_factory()
 *	Description : Create the device object(s)
 *                and store them in the device list
 */
//--------------------------------------------------------
void EbsCorrectorClass::device_factory(const Tango::DevVarStringArray *devlist_ptr)
{
	/*----- PROTECTED REGION ID(EbsCorrectorClass::device_factory_before) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	EbsCorrectorClass::device_factory_before

	//	Create devices and add it into the device list
	for (unsigned long i=0 ; i<devlist_ptr->length() ; i++)
	{
		cout4 << "Device name : " << (*devlist_ptr)[i].in() << endl;
		device_list.push_back(new EbsCorrector(this, (*devlist_ptr)[i]));
	}

	//	Manage dynamic attributes if any
	erase_dynamic_attributes(devlist_ptr, get_class_attr()->get_attr_list());

	//	Export devices to the outside world
	for (unsigned long i=1 ; i<=devlist_ptr->length() ; i++)
	{
		//	Add dynamic attributes if any
		EbsCorrector *dev = static_cast<EbsCorrector *>(device_list[device_list.size()-i]);
		dev->add_dynamic_attributes();
		dev->add_dynamic_commands();

		//	Check before if database used.
		if ((Tango::Util::_UseDb == true) && (Tango::Util::_FileDb == false))
			export_device(dev);
		else
			export_device(dev, dev->get_name().c_str());
	}

	/*----- PROTECTED REGION ID(EbsCorrectorClass::device_factory_after) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	EbsCorrectorClass::device_factory_after
}
//--------------------------------------------------------
/**
 *	Method      : EbsCorrectorClass::attribute_factory()
 *	Description : Create the attribute object(s)
 *                and store them in the attribute list
 */
//--------------------------------------------------------
void EbsCorrectorClass::attribute_factory(vector<Tango::Attr *> &att_list)
{
	/*----- PROTECTED REGION ID(EbsCorrectorClass::attribute_factory_before) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	EbsCorrectorClass::attribute_factory_before
	//	Attribute : Strength
	StrengthAttrib	*strength = new StrengthAttrib();
	Tango::UserDefaultAttrProp	strength_prop;
	//	description	not set for Strength
	//	label	not set for Strength
	//	unit	not set for Strength
	//	standard_unit	not set for Strength
	//	display_unit	not set for Strength
	strength_prop.set_format("%9.6f");
	//	max_value	not set for Strength
	//	min_value	not set for Strength
	//	max_alarm	not set for Strength
	//	min_alarm	not set for Strength
	//	max_warning	not set for Strength
	//	min_warning	not set for Strength
	//	delta_t	not set for Strength
	//	delta_val	not set for Strength
	
	strength->set_default_properties(strength_prop);
	//	Not Polled
	strength->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(strength);

	//	Attribute : Frozen
	FrozenAttrib	*frozen = new FrozenAttrib();
	Tango::UserDefaultAttrProp	frozen_prop;
	//	description	not set for Frozen
	//	label	not set for Frozen
	//	unit	not set for Frozen
	//	standard_unit	not set for Frozen
	//	display_unit	not set for Frozen
	//	format	not set for Frozen
	//	max_value	not set for Frozen
	//	min_value	not set for Frozen
	//	max_alarm	not set for Frozen
	//	min_alarm	not set for Frozen
	//	max_warning	not set for Frozen
	//	min_warning	not set for Frozen
	//	delta_t	not set for Frozen
	//	delta_val	not set for Frozen
	
	frozen->set_default_properties(frozen_prop);
	//	Not Polled
	frozen->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(frozen);


	//	Create a list of static attributes
	create_static_attribute_list(get_class_attr()->get_attr_list());
	/*----- PROTECTED REGION ID(EbsCorrectorClass::attribute_factory_after) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	EbsCorrectorClass::attribute_factory_after
}
//--------------------------------------------------------
/**
 *	Method      : EbsCorrectorClass::pipe_factory()
 *	Description : Create the pipe object(s)
 *                and store them in the pipe list
 */
//--------------------------------------------------------
void EbsCorrectorClass::pipe_factory()
{
	/*----- PROTECTED REGION ID(EbsCorrectorClass::pipe_factory_before) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	EbsCorrectorClass::pipe_factory_before
	/*----- PROTECTED REGION ID(EbsCorrectorClass::pipe_factory_after) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	EbsCorrectorClass::pipe_factory_after
}
//--------------------------------------------------------
/**
 *	Method      : EbsCorrectorClass::command_factory()
 *	Description : Create the command object(s)
 *                and store them in the command list
 */
//--------------------------------------------------------
void EbsCorrectorClass::command_factory()
{
	/*----- PROTECTED REGION ID(EbsCorrectorClass::command_factory_before) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	EbsCorrectorClass::command_factory_before


	//	Command SetpointCheck
	SetpointCheckClass	*pSetpointCheckCmd =
		new SetpointCheckClass("SetpointCheck",
			Tango::DEV_DOUBLE, Tango::DEV_BOOLEAN,
			"",
			"",
			Tango::OPERATOR);
	command_list.push_back(pSetpointCheckCmd);

	//	Command Reset
	ResetClass	*pResetCmd =
		new ResetClass("Reset",
			Tango::DEV_VOID, Tango::DEV_VOID,
			"",
			"",
			Tango::OPERATOR);
	command_list.push_back(pResetCmd);

	/*----- PROTECTED REGION ID(EbsCorrectorClass::command_factory_after) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	EbsCorrectorClass::command_factory_after
}

//===================================================================
//	Dynamic attributes related methods
//===================================================================

//--------------------------------------------------------
/**
 * method : 		EbsCorrectorClass::create_static_attribute_list
 * description : 	Create the a list of static attributes
 *
 * @param	att_list	the ceated attribute list
 */
//--------------------------------------------------------
void EbsCorrectorClass::create_static_attribute_list(vector<Tango::Attr *> &att_list)
{
	for (unsigned long i=0 ; i<att_list.size() ; i++)
	{
		string att_name(att_list[i]->get_name());
		transform(att_name.begin(), att_name.end(), att_name.begin(), ::tolower);
		defaultAttList.push_back(att_name);
	}

	cout2 << defaultAttList.size() << " attributes in default list" << endl;

	/*----- PROTECTED REGION ID(EbsCorrectorClass::create_static_att_list) ENABLED START -----*/
	
	/*----- PROTECTED REGION END -----*/	//	EbsCorrectorClass::create_static_att_list
}


//--------------------------------------------------------
/**
 * method : 		EbsCorrectorClass::erase_dynamic_attributes
 * description : 	delete the dynamic attributes if any.
 *
 * @param	devlist_ptr	the device list pointer
 * @param	list of all attributes
 */
//--------------------------------------------------------
void EbsCorrectorClass::erase_dynamic_attributes(const Tango::DevVarStringArray *devlist_ptr, vector<Tango::Attr *> &att_list)
{
	Tango::Util *tg = Tango::Util::instance();

	for (unsigned long i=0 ; i<devlist_ptr->length() ; i++)
	{
		Tango::DeviceImpl *dev_impl = tg->get_device_by_name(((string)(*devlist_ptr)[i]).c_str());
		EbsCorrector *dev = static_cast<EbsCorrector *> (dev_impl);

		vector<Tango::Attribute *> &dev_att_list = dev->get_device_attr()->get_attribute_list();
		vector<Tango::Attribute *>::iterator ite_att;
		for (ite_att=dev_att_list.begin() ; ite_att != dev_att_list.end() ; ++ite_att)
		{
			string att_name((*ite_att)->get_name_lower());
			if ((att_name == "state") || (att_name == "status"))
				continue;
			vector<string>::iterator ite_str = find(defaultAttList.begin(), defaultAttList.end(), att_name);
			if (ite_str == defaultAttList.end())
			{
				cout2 << att_name << " is a UNWANTED dynamic attribute for device " << (*devlist_ptr)[i] << endl;
				Tango::Attribute &att = dev->get_device_attr()->get_attr_by_name(att_name.c_str());
				dev->remove_attribute(att_list[att.get_attr_idx()], true, false);
				--ite_att;
			}
		}
	}
	/*----- PROTECTED REGION ID(EbsCorrectorClass::erase_dynamic_attributes) ENABLED START -----*/
	
	/*----- PROTECTED REGION END -----*/	//	EbsCorrectorClass::erase_dynamic_attributes
}

//--------------------------------------------------------
/**
 *	Method      : EbsCorrectorClass::get_attr_object_by_name()
 *	Description : returns Tango::Attr * object found by name
 */
//--------------------------------------------------------
Tango::Attr *EbsCorrectorClass::get_attr_object_by_name(vector<Tango::Attr *> &att_list, string attname)
{
	vector<Tango::Attr *>::iterator it;
	for (it=att_list.begin() ; it<att_list.end() ; ++it)
		if ((*it)->get_name()==attname)
			return (*it);
	//	Attr does not exist
	return NULL;
}


/*----- PROTECTED REGION ID(EbsCorrectorClass::Additional Methods) ENABLED START -----*/

/*----- PROTECTED REGION END -----*/	//	EbsCorrectorClass::Additional Methods
} //	namespace
