/*----- PROTECTED REGION ID(EbsCorrector::DynAttrUtils.cpp) ENABLED START -----*/
//=============================================================================
//
// file :        EbsCorrectorDynAttrUtils.cpp
//
// description : Dynamic attributes utilities file for the EbsCorrector class
//
// project :     
//
// This file is part of Tango device class.
// 
// Tango is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Tango is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
// 
//
// Copyright (C): 2022
//                European Synchrotron Radiation Facility
//                BP 220, Grenoble 38043
//                France
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================


#include <EbsCorrector.h>
#include <EbsCorrectorClass.h>

/*----- PROTECTED REGION END -----*/	//	EbsCorrector::DynAttrUtils.cpp

//================================================================
//  Attributes managed are:
//================================================================
//  CorrectionStrength  |  Tango::DevDouble	Scalar
//  ResonanceStrength   |  Tango::DevDouble	Scalar
//  Current             |  Tango::DevDouble	Scalar
//  SetCurrentsAverage  |  Tango::DevDouble	Scalar
//================================================================

//	For compatibility reason, this file (EbsCorrectorDynAttrUtils)
//	manage also the dynamic command utilities.
//================================================================
//  The following table gives the correspondence
//  between command and method names.
//
//  Command name            |  Method name
//================================================================
//  ResetResonanceStrength  |  reset_resonance_strength
//================================================================

namespace EbsCorrector_ns
{
//=============================================================
//	Add/Remove dynamic attribute methods
//=============================================================

//--------------------------------------------------------
/**
 *	Add a CorrectionStrength dynamic attribute.
 *
 *  parameter attname: attribute name to be cretated and added.
 */
//--------------------------------------------------------
void EbsCorrector::add_CorrectionStrength_dynamic_attribute(string attname)
{
	//	Attribute : CorrectionStrength
	CorrectionStrengthAttrib	*correctionstrength = new CorrectionStrengthAttrib(attname);
	Tango::UserDefaultAttrProp	correctionstrength_prop;
	//	description	not set for CorrectionStrength
	//	label	not set for CorrectionStrength
	correctionstrength_prop.set_unit("m-1");
	//	standard_unit	not set for CorrectionStrength
	//	display_unit	not set for CorrectionStrength
	correctionstrength_prop.set_format("%9.6f");
	//	max_value	not set for CorrectionStrength
	//	min_value	not set for CorrectionStrength
	//	max_alarm	not set for CorrectionStrength
	//	min_alarm	not set for CorrectionStrength
	//	max_warning	not set for CorrectionStrength
	//	min_warning	not set for CorrectionStrength
	//	delta_t	not set for CorrectionStrength
	//	delta_val	not set for CorrectionStrength
	
	/*----- PROTECTED REGION ID(EbsCorrector::att_CorrectionStrength_dynamic_attribute) ENABLED START -----*/
	
	
	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::att_CorrectionStrength_dynamic_attribute
	correctionstrength->set_default_properties(correctionstrength_prop);
	//	Not Polled
	correctionstrength->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	CorrectionStrength_data.insert(make_pair(attname, 0.0));
	add_attribute(correctionstrength);
}
//--------------------------------------------------------
/**
 *	remove a CorrectionStrength dynamic attribute.
 *
 *  parameter attname: attribute name to be removed.
 */
//--------------------------------------------------------
void EbsCorrector::remove_CorrectionStrength_dynamic_attribute(string attname)
{
	remove_attribute(attname, true, Tango::Util::instance()->_UseDb);
	map<string,Tango::DevDouble>::iterator ite;
    if ((ite=CorrectionStrength_data.find(attname))!=CorrectionStrength_data.end())
    {
    	/*----- PROTECTED REGION ID(EbsCorrector::remove_CorrectionStrength_dynamic_attribute) ENABLED START -----*/
    	
    	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::remove_CorrectionStrength_dynamic_attribute
		CorrectionStrength_data.erase(ite);
	}
}
//--------------------------------------------------------
/**
 *	Add a ResonanceStrength dynamic attribute.
 *
 *  parameter attname: attribute name to be cretated and added.
 */
//--------------------------------------------------------
void EbsCorrector::add_ResonanceStrength_dynamic_attribute(string attname)
{
	//	Attribute : ResonanceStrength
	ResonanceStrengthAttrib	*resonancestrength = new ResonanceStrengthAttrib(attname);
	Tango::UserDefaultAttrProp	resonancestrength_prop;
	//	description	not set for ResonanceStrength
	//	label	not set for ResonanceStrength
	resonancestrength_prop.set_unit("m-1");
	//	standard_unit	not set for ResonanceStrength
	//	display_unit	not set for ResonanceStrength
	resonancestrength_prop.set_format("%9.6f");
	//	max_value	not set for ResonanceStrength
	//	min_value	not set for ResonanceStrength
	//	max_alarm	not set for ResonanceStrength
	//	min_alarm	not set for ResonanceStrength
	//	max_warning	not set for ResonanceStrength
	//	min_warning	not set for ResonanceStrength
	//	delta_t	not set for ResonanceStrength
	//	delta_val	not set for ResonanceStrength
	
	/*----- PROTECTED REGION ID(EbsCorrector::att_ResonanceStrength_dynamic_attribute) ENABLED START -----*/
	
	
	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::att_ResonanceStrength_dynamic_attribute
	resonancestrength->set_default_properties(resonancestrength_prop);
	//	Not Polled
	resonancestrength->set_disp_level(Tango::OPERATOR);
	resonancestrength->set_memorized();
	resonancestrength->set_memorized_init(false);
	ResonanceStrength_data.insert(make_pair(attname, 0.0));
	add_attribute(resonancestrength);
}
//--------------------------------------------------------
/**
 *	remove a ResonanceStrength dynamic attribute.
 *
 *  parameter attname: attribute name to be removed.
 */
//--------------------------------------------------------
void EbsCorrector::remove_ResonanceStrength_dynamic_attribute(string attname)
{
	remove_attribute(attname, true, Tango::Util::instance()->_UseDb);
	map<string,Tango::DevDouble>::iterator ite;
    if ((ite=ResonanceStrength_data.find(attname))!=ResonanceStrength_data.end())
    {
    	/*----- PROTECTED REGION ID(EbsCorrector::remove_ResonanceStrength_dynamic_attribute) ENABLED START -----*/
    	
    	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::remove_ResonanceStrength_dynamic_attribute
		ResonanceStrength_data.erase(ite);
	}
}
//--------------------------------------------------------
/**
 *	Add a Current dynamic attribute.
 *
 *  parameter attname: attribute name to be cretated and added.
 */
//--------------------------------------------------------
void EbsCorrector::add_Current_dynamic_attribute(string attname)
{
	//	Attribute : Current
	CurrentAttrib	*current = new CurrentAttrib(attname);
	Tango::UserDefaultAttrProp	current_prop;
	current_prop.set_description("Steering pseudo current");
	current_prop.set_label("Pseudo Current");
	current_prop.set_unit("A");
	//	standard_unit	not set for Current
	//	display_unit	not set for Current
	current_prop.set_format("%9.6f");
	//	max_value	not set for Current
	//	min_value	not set for Current
	//	max_alarm	not set for Current
	//	min_alarm	not set for Current
	//	max_warning	not set for Current
	//	min_warning	not set for Current
	//	delta_t	not set for Current
	//	delta_val	not set for Current
	
	/*----- PROTECTED REGION ID(EbsCorrector::att_Current_dynamic_attribute) ENABLED START -----*/
	
	
	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::att_Current_dynamic_attribute
	current->set_default_properties(current_prop);
	//	Not Polled
	current->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	Current_data.insert(make_pair(attname, 0.0));
	add_attribute(current);
}
//--------------------------------------------------------
/**
 *	remove a Current dynamic attribute.
 *
 *  parameter attname: attribute name to be removed.
 */
//--------------------------------------------------------
void EbsCorrector::remove_Current_dynamic_attribute(string attname)
{
	remove_attribute(attname, true, Tango::Util::instance()->_UseDb);
	map<string,Tango::DevDouble>::iterator ite;
    if ((ite=Current_data.find(attname))!=Current_data.end())
    {
    	/*----- PROTECTED REGION ID(EbsCorrector::remove_Current_dynamic_attribute) ENABLED START -----*/
    	
    	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::remove_Current_dynamic_attribute
		Current_data.erase(ite);
	}
}
//--------------------------------------------------------
/**
 *	Add a SetCurrentsAverage dynamic attribute.
 *
 *  parameter attname: attribute name to be cretated and added.
 */
//--------------------------------------------------------
void EbsCorrector::add_SetCurrentsAverage_dynamic_attribute(string attname)
{
	//	Attribute : SetCurrentsAverage
	SetCurrentsAverageAttrib	*setcurrentsaverage = new SetCurrentsAverageAttrib(attname);
	Tango::UserDefaultAttrProp	setcurrentsaverage_prop;
	setcurrentsaverage_prop.set_description("Average AC pseudo current");
	setcurrentsaverage_prop.set_label("AC Pseudo Current (avg)");
	setcurrentsaverage_prop.set_unit("mA");
	//	standard_unit	not set for SetCurrentsAverage
	//	display_unit	not set for SetCurrentsAverage
	setcurrentsaverage_prop.set_format("%9.3f");
	//	max_value	not set for SetCurrentsAverage
	//	min_value	not set for SetCurrentsAverage
	//	max_alarm	not set for SetCurrentsAverage
	//	min_alarm	not set for SetCurrentsAverage
	//	max_warning	not set for SetCurrentsAverage
	//	min_warning	not set for SetCurrentsAverage
	//	delta_t	not set for SetCurrentsAverage
	//	delta_val	not set for SetCurrentsAverage
	
	/*----- PROTECTED REGION ID(EbsCorrector::att_SetCurrentsAverage_dynamic_attribute) ENABLED START -----*/
	
	
	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::att_SetCurrentsAverage_dynamic_attribute
	setcurrentsaverage->set_default_properties(setcurrentsaverage_prop);
	//	Not Polled
	setcurrentsaverage->set_disp_level(Tango::EXPERT);
	//	Not Memorized
	SetCurrentsAverage_data.insert(make_pair(attname, 0.0));
	add_attribute(setcurrentsaverage);
}
//--------------------------------------------------------
/**
 *	remove a SetCurrentsAverage dynamic attribute.
 *
 *  parameter attname: attribute name to be removed.
 */
//--------------------------------------------------------
void EbsCorrector::remove_SetCurrentsAverage_dynamic_attribute(string attname)
{
	remove_attribute(attname, true, Tango::Util::instance()->_UseDb);
	map<string,Tango::DevDouble>::iterator ite;
    if ((ite=SetCurrentsAverage_data.find(attname))!=SetCurrentsAverage_data.end())
    {
    	/*----- PROTECTED REGION ID(EbsCorrector::remove_SetCurrentsAverage_dynamic_attribute) ENABLED START -----*/
    	
    	/*----- PROTECTED REGION END -----*/	//	EbsCorrector::remove_SetCurrentsAverage_dynamic_attribute
		SetCurrentsAverage_data.erase(ite);
	}
}


//============================================================
//	Tool methods to get pointer on attribute data buffer 
//============================================================
//--------------------------------------------------------
/**
 *	Return a pointer on CorrectionStrength data.
 *
 *  parameter attname: the specified attribute name.
 */
//--------------------------------------------------------
Tango::DevDouble *EbsCorrector::get_CorrectionStrength_data_ptr(string &name)
{
	map<string,Tango::DevDouble>::iterator ite;
    if ((ite=CorrectionStrength_data.find(name))==CorrectionStrength_data.end())
    {
		TangoSys_OMemStream	tms;
		tms << "Dynamic attribute " << name << " has not been created";
		Tango::Except::throw_exception(
					(const char *)"ATTRIBUTE_NOT_FOUND",
					tms.str().c_str(),
					(const char *)"EbsCorrector::get_CorrectionStrength_data_ptr()");
    }
	return  &(ite->second);
}
//--------------------------------------------------------
/**
 *	Return a pointer on ResonanceStrength data.
 *
 *  parameter attname: the specified attribute name.
 */
//--------------------------------------------------------
Tango::DevDouble *EbsCorrector::get_ResonanceStrength_data_ptr(string &name)
{
	map<string,Tango::DevDouble>::iterator ite;
    if ((ite=ResonanceStrength_data.find(name))==ResonanceStrength_data.end())
    {
		TangoSys_OMemStream	tms;
		tms << "Dynamic attribute " << name << " has not been created";
		Tango::Except::throw_exception(
					(const char *)"ATTRIBUTE_NOT_FOUND",
					tms.str().c_str(),
					(const char *)"EbsCorrector::get_ResonanceStrength_data_ptr()");
    }
	return  &(ite->second);
}
//--------------------------------------------------------
/**
 *	Return a pointer on Current data.
 *
 *  parameter attname: the specified attribute name.
 */
//--------------------------------------------------------
Tango::DevDouble *EbsCorrector::get_Current_data_ptr(string &name)
{
	map<string,Tango::DevDouble>::iterator ite;
    if ((ite=Current_data.find(name))==Current_data.end())
    {
		TangoSys_OMemStream	tms;
		tms << "Dynamic attribute " << name << " has not been created";
		Tango::Except::throw_exception(
					(const char *)"ATTRIBUTE_NOT_FOUND",
					tms.str().c_str(),
					(const char *)"EbsCorrector::get_Current_data_ptr()");
    }
	return  &(ite->second);
}
//--------------------------------------------------------
/**
 *	Return a pointer on SetCurrentsAverage data.
 *
 *  parameter attname: the specified attribute name.
 */
//--------------------------------------------------------
Tango::DevDouble *EbsCorrector::get_SetCurrentsAverage_data_ptr(string &name)
{
	map<string,Tango::DevDouble>::iterator ite;
    if ((ite=SetCurrentsAverage_data.find(name))==SetCurrentsAverage_data.end())
    {
		TangoSys_OMemStream	tms;
		tms << "Dynamic attribute " << name << " has not been created";
		Tango::Except::throw_exception(
					(const char *)"ATTRIBUTE_NOT_FOUND",
					tms.str().c_str(),
					(const char *)"EbsCorrector::get_SetCurrentsAverage_data_ptr()");
    }
	return  &(ite->second);
}


//=============================================================
//	Add/Remove dynamic command methods
//=============================================================

//--------------------------------------------------------
/**
 *	Add a ResetResonanceStrength dynamic command.
 *
 *  parameter cmdname: command name to be cretated and added.
 *  parameter device:  Set this flag to true if the command must be added for only this device.
 */
//--------------------------------------------------------
void EbsCorrector::add_ResetResonanceStrength_dynamic_command(string cmdname, bool device)
{
	ResetResonanceStrengthClass	*pResetResonanceStrengthCmd =
		new ResetResonanceStrengthClass(cmdname.c_str(),
			Tango::DEV_VOID, Tango::DEV_VOID,
			"",
			"",
			Tango::OPERATOR);
	add_command(pResetResonanceStrengthCmd, device);
}
//--------------------------------------------------------
/**
 *	remove a ResetResonanceStrength dynamic command.
 *
 *  parameter cmdname: command name to be removed.
 */
//--------------------------------------------------------
void EbsCorrector::remove_ResetResonanceStrength_dynamic_command(string cmdname)
{
	remove_command(cmdname, true);
}

} //	namespace
