/*----- PROTECTED REGION ID(EbsMagnet.cpp) ENABLED START -----*/
//=============================================================================
//
// file :        EbsMagnet.cpp
//
// description : C++ source for the EbsMagnet class and its commands.
//               The class is derived from Device. It represents the
//               CORBA servant object which will be accessed from the
//               network. All commands which can be executed on the
//               EbsMagnet are implemented in this file.
//
// project :     EbsMagnet
//
// This file is part of Tango device class.
// 
// Tango is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Tango is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
// 
//
// Copyright (C): 2022
//                European Synchrotron Radiation Facility
//                BP 220, Grenoble 38043
//                France
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================


#include <EbsMagnet.h>
#include <EbsMagnetClass.h>
#include <EbsCorrector.h>

/*----- PROTECTED REGION END -----*/	//	EbsMagnet.cpp

/**
 *  EbsMagnet class description:
 *    Class to handle EBS magnets
 */

//================================================================
//  The following table gives the correspondence
//  between command and method names.
//
//  Command name              |  Method name
//================================================================
//  State                     |  dev_state
//  Status                    |  Inherited (no method)
//  Reset                     |  reset
//  SetDesignStrength         |  set_design_strength
//  ApplyDeltaStrength        |  apply_delta_strength
//  ResetResonanceStrength    |  reset_resonance_strength
//  MainCurrentSetpointCheck  |  main_current_setpoint_check
//  On                        |  on
//  Off                       |  off
//  GetACPSCurrent            |  get_acpscurrent
//  GetPSCurrent              |  get_pscurrent
//  WritePSCurrent            |  write_pscurrent
//  Cycle                     |  cycle
//  Swap                      |  swap
//  Unswap                    |  unswap
//================================================================

//================================================================
//  Attributes managed are:
//================================================================
//  SerialNumber    |  Tango::DevString	Scalar
//  CorrectorState  |  Tango::DevState	Scalar
//  RequireCycling  |  Tango::DevBoolean	Scalar
//  Currents        |  Tango::DevDouble	Spectrum  ( max = 5)
//  Strengths       |  Tango::DevDouble	Spectrum  ( max = 5)
//================================================================

namespace EbsMagnet_ns
{
/*----- PROTECTED REGION ID(EbsMagnet::namespace_starting) ENABLED START -----*/

//	static initializations

/*----- PROTECTED REGION END -----*/	//	EbsMagnet::namespace_starting

//--------------------------------------------------------
/**
 *	Method      : EbsMagnet::EbsMagnet()
 *	Description : Constructors for a Tango device
 *                implementing the classEbsMagnet
 */
//--------------------------------------------------------
EbsMagnet::EbsMagnet(Tango::DeviceClass *cl, string &s)
 : TANGO_BASE_CLASS(cl, s.c_str())
{
	/*----- PROTECTED REGION ID(EbsMagnet::constructor_1) ENABLED START -----*/
  magnet = nullptr;
	init_device();
	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::constructor_1
}
//--------------------------------------------------------
EbsMagnet::EbsMagnet(Tango::DeviceClass *cl, const char *s)
 : TANGO_BASE_CLASS(cl, s)
{
	/*----- PROTECTED REGION ID(EbsMagnet::constructor_2) ENABLED START -----*/
  magnet = nullptr;
	init_device();
	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::constructor_2
}
//--------------------------------------------------------
EbsMagnet::EbsMagnet(Tango::DeviceClass *cl, const char *s, const char *d)
 : TANGO_BASE_CLASS(cl, s, d)
{
	/*----- PROTECTED REGION ID(EbsMagnet::constructor_3) ENABLED START -----*/
  magnet = nullptr;
	init_device();
	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::constructor_3
}

//--------------------------------------------------------
/**
 *	Method      : EbsMagnet::delete_device()
 *	Description : will be called at device destruction or at init command
 */
//--------------------------------------------------------
void EbsMagnet::delete_device()
{
	DEBUG_STREAM << "EbsMagnet::delete_device() " << device_name << endl;
	/*----- PROTECTED REGION ID(EbsMagnet::delete_device) ENABLED START -----*/

	//	Delete device allocated objects
	delete psGroup;

	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::delete_device
	delete[] attr_SerialNumber_read;
	delete[] attr_CorrectorState_read;
	delete[] attr_RequireCycling_read;
	delete[] attr_Currents_read;
	delete[] attr_Strengths_read;
}

//--------------------------------------------------------
/**
 *	Method      : EbsMagnet::init_device()
 *	Description : will be called at device initialization.
 */
//--------------------------------------------------------
void EbsMagnet::init_device()
{
	DEBUG_STREAM << "EbsMagnet::init_device() create device " << device_name << endl;
	/*----- PROTECTED REGION ID(EbsMagnet::init_device_before) ENABLED START -----*/
	
	//	Initialization before get_device_property() call
  psGroup = nullptr;

	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::init_device_before
	

	//	Get the device properties from database
	get_device_property();
	
	attr_SerialNumber_read = new Tango::DevString[1];
	attr_CorrectorState_read = new Tango::DevState[1];
	attr_RequireCycling_read = new Tango::DevBoolean[1];
	attr_Currents_read = new Tango::DevDouble[5];
	attr_Strengths_read = new Tango::DevDouble[5];
	/*----- PROTECTED REGION ID(EbsMagnet::init_device) ENABLED START -----*/
	
	//	Check properties
  if(model.empty()) {
    cerr << "Fatal Error: Model property not defined for " << get_name() << endl;
    exit(0);
  }
  if(type.empty()) {
    cerr << "Fatal Error: Type property not defined for " << get_name() << endl;
    exit(0);
  }
  if(currentDevices.empty()) {
    cerr << "Fatal Error: CurrentDevices property not defined for " << get_name() << endl;
    exit(0);
  }

  // Create the Magnet
  if(!magnet) magnet = Magnet::Create(this,focusing,crossTalkFactor,serialNumber,type,model);
  magnet->setDesignStrength(designStrength);
  setCurrents.assign(magnet->getCurrentNumber(),NAN);
  currents.assign(magnet->getCurrentNumber(),NAN);
  avgACPseudoCurrents.assign(magnet->getStrengthNumber(),NAN);
  pseudoCurrents.assign(magnet->getStrengthNumber(),NAN);
  setPseudoCurrents.assign(magnet->getStrengthNumber(),NAN);
  setCurrentsMin.assign(magnet->getCurrentNumber(),NAN);
  setCurrentsMax.assign(magnet->getCurrentNumber(),NAN);
  psStates.assign(magnet->getCurrentNumber(),Tango::UNKNOWN);
  psStatus.assign(magnet->getCurrentNumber(),"Not initialized");
  attr_CorrectorState_read[0] = Tango::UNKNOWN;
  minmaxAssigned = false;
  isSH = Magnet::compare(type, "SH3") || Magnet::compare(type, "SH5");

  if(currentDevices.size() != magnet->getCurrentNumber()) {
    cerr << "Fatal Error: " << get_name() << ": CurrentDevices property " << magnet->getCurrentNumber() << " items expected" << endl;
    exit(0);
  }


  Tango::WAttribute &watt = get_device_attr()->get_w_attr_by_name("Currents");
  currentsAtt = &watt;

  // Build power supplies group
  Tango::GroupAttrReply::enable_exception(false);
  psGroup = new Tango::Group("PS");
  for(auto& dveName:currentDevices)
    psGroup->add(currentDevices);
  if(psGroup->get_size()!=currentDevices.size()) {
    cerr << "Fatal Error: " << get_name() << ": One or more currentDevices is not defined" << endl;
    exit(0);
  }
  // Force device call (needed to avoid wrong setpoint due to eventual polling)
  for(size_t i=0;i<psGroup->get_size();i++)
    psGroup->get_device(i+1)->set_source(Tango::DEV); 
  psTimeout = 3800; // MagnetType and CorrectorType have a 4 sec timeout
  psGroup->set_timeout_millis(psTimeout);

  attr_RequireCycling_read[0] = false;

	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::init_device
}

//--------------------------------------------------------
/**
 *	Method      : EbsMagnet::get_device_property()
 *	Description : Read database to initialize property data members.
 */
//--------------------------------------------------------
void EbsMagnet::get_device_property()
{
	/*----- PROTECTED REGION ID(EbsMagnet::get_device_property_before) ENABLED START -----*/
	
	//	Initialize property data members
  crossTalkFactor = 1.0;
  model = "";
  type = "";
  serialNumber= "None";
  focusing = false;
  currentDevices.clear();
  designStrength = 0.0;
	
	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::get_device_property_before


	//	Read device properties from database.
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("CrossTalkFactor"));
	dev_prop.push_back(Tango::DbDatum("Model"));
	dev_prop.push_back(Tango::DbDatum("Type"));
	dev_prop.push_back(Tango::DbDatum("SerialNumber"));
	dev_prop.push_back(Tango::DbDatum("Focusing"));
	dev_prop.push_back(Tango::DbDatum("CurrentDevices"));
	dev_prop.push_back(Tango::DbDatum("DesignStrength"));
	dev_prop.push_back(Tango::DbDatum("MainCurrentMax"));

	//	is there at least one property to be read ?
	if (dev_prop.size()>0)
	{
		//	Call database and extract values
		if (Tango::Util::instance()->_UseDb==true)
			get_db_device()->get_property(dev_prop);
	
		//	get instance on EbsMagnetClass to get class property
		Tango::DbDatum	def_prop, cl_prop;
		EbsMagnetClass	*ds_class =
			(static_cast<EbsMagnetClass *>(get_device_class()));
		int	i = -1;

		//	Try to initialize CrossTalkFactor from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  crossTalkFactor;
		else {
			//	Try to initialize CrossTalkFactor from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  crossTalkFactor;
		}
		//	And try to extract CrossTalkFactor value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  crossTalkFactor;

		//	Try to initialize Model from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  model;
		else {
			//	Try to initialize Model from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  model;
		}
		//	And try to extract Model value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  model;

		//	Try to initialize Type from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  type;
		else {
			//	Try to initialize Type from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  type;
		}
		//	And try to extract Type value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  type;

		//	Try to initialize SerialNumber from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  serialNumber;
		else {
			//	Try to initialize SerialNumber from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  serialNumber;
		}
		//	And try to extract SerialNumber value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  serialNumber;

		//	Try to initialize Focusing from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  focusing;
		else {
			//	Try to initialize Focusing from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  focusing;
		}
		//	And try to extract Focusing value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  focusing;

		//	Try to initialize CurrentDevices from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  currentDevices;
		else {
			//	Try to initialize CurrentDevices from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  currentDevices;
		}
		//	And try to extract CurrentDevices value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  currentDevices;

		//	Try to initialize DesignStrength from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  designStrength;
		else {
			//	Try to initialize DesignStrength from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  designStrength;
		}
		//	And try to extract DesignStrength value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  designStrength;

		//	Try to initialize MainCurrentMax from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  mainCurrentMax;
		else {
			//	Try to initialize MainCurrentMax from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  mainCurrentMax;
		}
		//	And try to extract MainCurrentMax value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  mainCurrentMax;

	}

	/*----- PROTECTED REGION ID(EbsMagnet::get_device_property_after) ENABLED START -----*/
	
	//	Check device property data members init
	
	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::get_device_property_after
}

//--------------------------------------------------------
/**
 *	Method      : EbsMagnet::always_executed_hook()
 *	Description : method always executed before any command is executed
 */
//--------------------------------------------------------
void EbsMagnet::always_executed_hook()
{
	DEBUG_STREAM << "EbsMagnet::always_executed_hook()  " << device_name << endl;
	/*----- PROTECTED REGION ID(EbsMagnet::always_executed_hook) ENABLED START -----*/
	
	//	code always executed before all requests
	
	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::always_executed_hook
}

//--------------------------------------------------------
/**
 *	Method      : EbsMagnet::read_attr_hardware()
 *	Description : Hardware acquisition for attributes
 */
//--------------------------------------------------------
void EbsMagnet::read_attr_hardware(TANGO_UNUSED(vector<long> &attr_list))
{
	DEBUG_STREAM << "EbsMagnet::read_attr_hardware(vector<long> &attr_list) entering... " << endl;
	/*----- PROTECTED REGION ID(EbsMagnet::read_attr_hardware) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::read_attr_hardware
}
//--------------------------------------------------------
/**
 *	Method      : EbsMagnet::write_attr_hardware()
 *	Description : Hardware writing for attributes
 */
//--------------------------------------------------------
void EbsMagnet::write_attr_hardware(TANGO_UNUSED(vector<long> &attr_list))
{
	DEBUG_STREAM << "EbsMagnet::write_attr_hardware(vector<long> &attr_list) entering... " << endl;
	/*----- PROTECTED REGION ID(EbsMagnet::write_attr_hardware) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::write_attr_hardware
}

//--------------------------------------------------------
/**
 *	Read attribute SerialNumber related method
 *	Description: 
 *
 *	Data type:	Tango::DevString
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void EbsMagnet::read_SerialNumber(Tango::Attribute &attr)
{
	DEBUG_STREAM << "EbsMagnet::read_SerialNumber(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(EbsMagnet::read_SerialNumber) ENABLED START -----*/
  attr_SerialNumber_read[0] = (Tango::DevString)serialNumber.c_str();
	attr.set_value(attr_SerialNumber_read);
	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::read_SerialNumber
}
//--------------------------------------------------------
/**
 *	Read attribute CorrectorState related method
 *	Description: 
 *
 *	Data type:	Tango::DevState
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void EbsMagnet::read_CorrectorState(Tango::Attribute &attr)
{
	DEBUG_STREAM << "EbsMagnet::read_CorrectorState(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(EbsMagnet::read_CorrectorState) ENABLED START -----*/
	attr.set_value(attr_CorrectorState_read);
	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::read_CorrectorState
}
//--------------------------------------------------------
/**
 *	Read attribute RequireCycling related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void EbsMagnet::read_RequireCycling(Tango::Attribute &attr)
{
	DEBUG_STREAM << "EbsMagnet::read_RequireCycling(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(EbsMagnet::read_RequireCycling) ENABLED START -----*/
	//	Set the attribute value
	attr.set_value(attr_RequireCycling_read);
	
	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::read_RequireCycling
}
//--------------------------------------------------------
/**
 *	Read attribute Currents related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Spectrum max = 5
 */
//--------------------------------------------------------
void EbsMagnet::read_Currents(Tango::Attribute &attr)
{
	DEBUG_STREAM << "EbsMagnet::read_Currents(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(EbsMagnet::read_Currents) ENABLED START -----*/
  for(size_t i=0;i<currents.size();i++)
    attr_Currents_read[i]=currents[i];
	attr.set_value(attr_Currents_read, currents.size());
	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::read_Currents
}
//--------------------------------------------------------
/**
 *	Write attribute Currents related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Spectrum max = 5
 */
//--------------------------------------------------------
void EbsMagnet::write_Currents(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "EbsMagnet::write_Currents(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve number of write values
	int	w_length = attr.get_write_value_length();

	//	Retrieve pointer on write values (Do not delete !)
	const Tango::DevDouble	*w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(EbsMagnet::write_Currents) ENABLED START -----*/

	if(w_length != setCurrents.size()) {
    Tango::Except::throw_exception("ErrorWrite",
                                   "Invalid number of current(s), " + std::to_string(setCurrents.size()) + " item(s) expected" ,
                                   "EbsMagnet::write_Currents");
  }

  for(size_t i=0;i<setCurrents.size();i++)
    setCurrents[i] = w_val[i];
  magnet->send_currents();

	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::write_Currents
}
//--------------------------------------------------------
/**
 *	Read attribute Strengths related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Spectrum max = 5
 */
//--------------------------------------------------------
void EbsMagnet::read_Strengths(Tango::Attribute &attr)
{
	DEBUG_STREAM << "EbsMagnet::read_Strengths(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(EbsMagnet::read_Strengths) ENABLED START -----*/

  magnet->readStrengths(attr_Strengths_read);
	attr.set_value(attr_Strengths_read, magnet->getStrengthNumber());
	
	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::read_Strengths
}
//--------------------------------------------------------
/**
 *	Write attribute Strengths related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Spectrum max = 5
 */
//--------------------------------------------------------
void EbsMagnet::write_Strengths(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "EbsMagnet::write_Strengths(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve number of write values
	int	w_length = attr.get_write_value_length();

	//	Retrieve pointer on write values (Do not delete !)
	const Tango::DevDouble	*w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(EbsMagnet::write_Strengths) ENABLED START -----*/

  if( w_length != magnet->getStrengthNumber() )
       Tango::Except::throw_exception(
          "ErrorWrite",
          "Unexpected number of strength, " + to_string(magnet->getStrengthNumber()) + " expected, but got " +
                  to_string(w_length),
          "Magnet::main_current_setpoint_check");

  vector<double> s;
  s.assign(w_val,w_val+w_length);
  magnet->writeStrengths(s);

	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::write_Strengths
}

//--------------------------------------------------------
/**
 *	Method      : EbsMagnet::add_dynamic_attributes()
 *	Description : Create the dynamic attributes if any
 *                for specified device.
 */
//--------------------------------------------------------
void EbsMagnet::add_dynamic_attributes()
{
	/*----- PROTECTED REGION ID(EbsMagnet::add_dynamic_attributes) ENABLED START -----*/

  auto atts = magnet->getAttributes();
  for(auto &att:atts)
    add_attribute(att);

  // Update ResonanceStrength setpoint
  if(magnet->hasMainCurrent()) {
    Tango::WAttribute &resAtt = get_device_attr()->get_w_attr_by_name("ResonanceStrength");
    try {
      magnet->setResonanceStrength(std::stod(resAtt.get_mem_value()));
    } catch (std::invalid_argument &e) {
      magnet->setResonanceStrength(0.0);
    }
  }

	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::add_dynamic_attributes
}

//--------------------------------------------------------
/**
 *	Command State related method
 *	Description: This command gets the device state (stored in its device_state data member) and returns it to the caller.
 *
 *	@returns Device state
 */
//--------------------------------------------------------
Tango::DevState EbsMagnet::dev_state()
{
	DEBUG_STREAM << "EbsMagnet::State()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(EbsMagnet::dev_state) ENABLED START -----*/

  Tango::DevState argout = Tango::ON;
  string status;
  status.reserve(4096);

  // Read power supplies states/currents
  time_t t0 = get_ticks();
  read_ps();
  time_t t1 = get_ticks();
  if((t1-t0)>1500) {
    cout << get_name() << ": read_ps() " << (t1-t0) << "ms" << endl;
  }

  // Main status
  if (magnet->hasMainCurrent()) {
    argout = psStates[0];
    status.append(currentDevices[0]);
    status.append(": ");
    status.append(Tango::DevStateName[psStates[0]]);
    status.append("\n");
    status.append(psStatus[0]);
    status.append("\n\n");
  }

  if(magnet->isMainFrozen()) {
    argout = Tango::DISABLE;
    status = "Magnet is frozen\n";
  }

  // Correctors
  update_corrector_state();
  for(size_t i=(magnet->hasMainCurrent()?1:0);i<psStates.size();i++) {
    status.append(currentDevices[i]);
    status.append(": ");
    status.append(Tango::DevStateName[psStates[i]]);
    status.append("\n");
    if(psStates[i]==Tango::FAULT || psStates[i]==Tango::UNKNOWN) {
      status.append(psStatus[i]);
      status.append("\n");
    }
  }

  if (magnet->hasMainCurrent() && argout == Tango::ON && attr_CorrectorState_read[0] != Tango::ON) {
    // Alarm when corrector PS are not ON and main PS is ON
    argout = Tango::ALARM;
  }

  // Update strengths (read and set value) from currents
  magnet->setCurrentsAndUpdateStrength(currents, setCurrents);
  // Update currents write_value
  read_minmax();
  // Update tango setpoint
  currentsAtt->set_write_value(setCurrents,setCurrents.size());

  set_status(status);
	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::dev_state
	set_state(argout);    // Give the state to Tango.
	if (argout!=Tango::ALARM)
		Tango::DeviceImpl::dev_state();
	return get_state();  // Return it after Tango management.
}
//--------------------------------------------------------
/**
 *	Command Reset related method
 *	Description: Reset underlying power supplies
 *
 */
//--------------------------------------------------------
void EbsMagnet::reset()
{
	DEBUG_STREAM << "EbsMagnet::Reset()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(EbsMagnet::reset) ENABLED START -----*/
  exec_command("Reset");
	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::reset
}
//--------------------------------------------------------
/**
 *	Command SetDesignStrength related method
 *	Description: Set the design strength (Expert usage)
 *               (DesignStrength has to be a read only attribute)
 *
 *	@param argin Design strength
 */
//--------------------------------------------------------
void EbsMagnet::set_design_strength(Tango::DevDouble argin)
{
	DEBUG_STREAM << "EbsMagnet::SetDesignStrength()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(EbsMagnet::set_design_strength) ENABLED START -----*/

  magnet->setDesignStrength(argin);
  Tango::DbDatum dis("DesignStrength");
  dis << argin;
  Tango::DbData db_d{dis};
  get_db_device()->put_property(db_d);

	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::set_design_strength
}
//--------------------------------------------------------
/**
 *	Command ApplyDeltaStrength related method
 *	Description: Apply delta strength
 *
 *	@param argin Delta strength to be applied
 */
//--------------------------------------------------------
void EbsMagnet::apply_delta_strength(Tango::DevDouble argin)
{
	DEBUG_STREAM << "EbsMagnet::ApplyDeltaStrength()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(EbsMagnet::apply_delta_strength) ENABLED START -----*/
  if( isnan(argin) )
    return;

  if( magnet->hasMainCurrent() ) {
    double newSet = magnet->getCorrectionStrength() + argin;
    magnet->writeAttribute(newSet,"CorrectionStrength",currentDevices,psStates);
  }

	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::apply_delta_strength
}
//--------------------------------------------------------
/**
 *	Command ResetResonanceStrength related method
 *	Description: Set resonnance strength to 0
 *
 */
//--------------------------------------------------------
void EbsMagnet::reset_resonance_strength()
{
	DEBUG_STREAM << "EbsMagnet::ResetResonanceStrength()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(EbsMagnet::reset_resonance_strength) ENABLED START -----*/

  if(magnet->hasMainCurrent() && magnet->getResonanceStrength()!=0.0 ) {
    magnet->resetResonanceStrength();
    get_device_attr()->get_w_attr_by_name("ResonanceStrength").set_write_value((Tango::DevDouble) 0);
    // Save to DB
    Tango::Database *db = Tango::Util::instance()->get_database();
    Tango::DbDatum reso("ResonanceStrength");
    reso << 1;
    Tango::DbDatum val("__value");
    val << 0.0;
    Tango::DbData db_data = {reso, val};
    db->put_device_attribute_property(get_name(), db_data);
  }

	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::reset_resonance_strength
}
//--------------------------------------------------------
/**
 *	Command MainCurrentSetpointCheck related method
 *	Description: Check if current setpoint of the main power supply is valid.
 *               If the magnet hasn`t a main power supply, returns true.
 *
 *	@param argin New main current
 *	@returns True if PS ia able to generate the required current
 */
//--------------------------------------------------------
Tango::DevBoolean EbsMagnet::main_current_setpoint_check(Tango::DevDouble argin)
{
	Tango::DevBoolean argout;
	DEBUG_STREAM << "EbsMagnet::MainCurrentSetpointCheck()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(EbsMagnet::main_current_setpoint_check) ENABLED START -----*/
  if(!minmaxAssigned)
    Tango::Except::throw_exception(
            "ErrorCommand",
            "min/max current(s) of underlying power supply(ies) not updated",
            "Magnet::main_current_setpoint_check");

  if(magnet->hasMainCurrent()) {
    argout = (setCurrentsMin[0] <= argin) && (setCurrentsMax[0] >= argin);
  } else {
    argout = true;
  }

	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::main_current_setpoint_check
	return argout;
}
//--------------------------------------------------------
/**
 *	Command On related method
 *	Description: Switch ON underlying power supply
 *
 */
//--------------------------------------------------------
void EbsMagnet::on()
{
	DEBUG_STREAM << "EbsMagnet::On()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(EbsMagnet::on) ENABLED START -----*/
  exec_command("On");
	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::on
}
//--------------------------------------------------------
/**
 *	Command Off related method
 *	Description: Switch OFF underlying power supply
 *
 */
//--------------------------------------------------------
void EbsMagnet::off()
{
	DEBUG_STREAM << "EbsMagnet::Off()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(EbsMagnet::off) ENABLED START -----*/
  exec_command("Off");
	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::off
}
//--------------------------------------------------------
/**
 *	Command GetACPSCurrent related method
 *	Description: Get the SH AC pseudo current (0=>h 1=>v 2=>sq)
 *               If the magnet is not a SH, returns NAN
 *
 *	@param argin pseudo current index (0=>h 1=>v 2=>sq)
 *	@returns Pseudo current
 */
//--------------------------------------------------------
Tango::DevDouble EbsMagnet::get_acpscurrent(Tango::DevShort argin)
{
	Tango::DevDouble argout;
	DEBUG_STREAM << "EbsMagnet::GetACPSCurrent()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(EbsMagnet::get_acpscurrent) ENABLED START -----*/
  if(isSH && (argin>=0) && argin<magnet->getStrengthNumber()) {
  	argout = avgACPseudoCurrents[argin];
  } else {
    argout = NAN;
  }
	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::get_acpscurrent
	return argout;
}
//--------------------------------------------------------
/**
 *	Command GetPSCurrent related method
 *	Description: Get the SH pseudo current (0=>h 1=>v 2=>sq)
 *               If the magnet is not a SH, returns NAN
 *
 *	@param argin Pseudo current index (0=>h 1=>v 2=>sq)
 *	@returns Pseudo current (read,set)
 */
//--------------------------------------------------------
Tango::DevVarDoubleArray *EbsMagnet::get_pscurrent(Tango::DevShort argin)
{
	Tango::DevVarDoubleArray *argout;
	DEBUG_STREAM << "EbsMagnet::GetPSCurrent()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(EbsMagnet::get_pscurrent) ENABLED START -----*/
  argout = new Tango::DevVarDoubleArray();
  argout->length(2);
  if(isSH && (argin>=0) && argin<magnet->getStrengthNumber()) {
  	(*argout)[0] = pseudoCurrents[argin];
  	(*argout)[1] = setPseudoCurrents[argin];
  } else {
  	(*argout)[0] = NAN;
  	(*argout)[1] = NAN;
  }
	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::get_pscurrent
	return argout;
}
//--------------------------------------------------------
/**
 *	Command WritePSCurrent related method
 *	Description: Write SH pseudo current.
 *               argin[0] = idx (0=>h 1=>v 2=>sq)
 *               argin[1] = ps current
 *               If the magnet is not a SH, the command has no effect.
 *
 *	@param argin [0]=Pseuco current idx,[1]=Pseudo current
 */
//--------------------------------------------------------
void EbsMagnet::write_pscurrent(const Tango::DevVarDoubleArray *argin)
{
	DEBUG_STREAM << "EbsMagnet::WritePSCurrent()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(EbsMagnet::write_pscurrent) ENABLED START -----*/
	
  // Compute corresponding strength and apply it
  // This is mainly used by the FOFB fast offset reinjection loop
  if(!isSH)
    return;
  
  int idx = (int)(*argin)[0];  
  double psCurrent = (*argin)[1];
  double strength;
  magnet->computeStrengthFromPseudoCurrent(idx,psCurrent,strength);
  switch (idx)
  {
    case 0:
    magnet->writeAttribute(strength, "Strength_H",currentDevices,psStates);
    break;
    case 1:
    magnet->writeAttribute(strength, "Strength_V",currentDevices,psStates);
    break;
    case 2:
    magnet->writeAttribute(strength, "Strength_SQ",currentDevices,psStates);
    break;  
    default:
    Tango::Except::throw_exception(
            "ErrorWrite",
            "Unexpected pseudo current index " + std::to_string(idx) + " not in [0..2]",
            "EbsMagnet::write_pscurrent");
    break;
  }
	
	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::write_pscurrent
}
//--------------------------------------------------------
/**
 *	Command Cycle related method
 *	Description: Cycle the magnet
 *
 */
//--------------------------------------------------------
void EbsMagnet::cycle()
{
	DEBUG_STREAM << "EbsMagnet::Cycle()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(EbsMagnet::cycle) ENABLED START -----*/

  if(magnet->hasMainCurrent()) {
    Tango::DeviceProxy *mainPS = psGroup->get_device(1);
    mainPS->command_inout("CycleSingle");
  }

	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::cycle
}
//--------------------------------------------------------
/**
 *	Command Swap related method
 *	Description: Send SWAP to HSM
 *
 */
//--------------------------------------------------------
void EbsMagnet::swap()
{
	DEBUG_STREAM << "EbsMagnet::Swap()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(EbsMagnet::swap) ENABLED START -----*/
  if(magnet->hasMainCurrent()) {
    Tango::DeviceProxy *mainPS = psGroup->get_device(1);
    mainPS->command_inout("Swap");
  }
	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::swap
}
//--------------------------------------------------------
/**
 *	Command Unswap related method
 *	Description: Send UNSWAP to HSM
 *
 */
//--------------------------------------------------------
void EbsMagnet::unswap()
{
	DEBUG_STREAM << "EbsMagnet::Unswap()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(EbsMagnet::unswap) ENABLED START -----*/
  if(magnet->hasMainCurrent()) {
    Tango::DeviceProxy *mainPS = psGroup->get_device(1);
    mainPS->command_inout("Unswap");
  }
	/*----- PROTECTED REGION END -----*/	//	EbsMagnet::unswap
}
//--------------------------------------------------------
/**
 *	Method      : EbsMagnet::add_dynamic_commands()
 *	Description : Create the dynamic commands if any
 *                for specified device.
 */
//--------------------------------------------------------
void EbsMagnet::add_dynamic_commands()
{
	/*----- PROTECTED REGION ID(EbsMagnet::add_dynamic_commands) ENABLED START -----*/

  auto cmds = magnet->getSetpointCheckCommands();
  for(auto &cmd:cmds)
    add_command(new SetpointCheckCommand(cmd.c_str(),Tango::EXPERT));

  /*----- PROTECTED REGION END -----*/	//	EbsMagnet::add_dynamic_commands
}

/*----- PROTECTED REGION ID(EbsMagnet::namespace_ending) ENABLED START -----*/

//	Additional Methods
void EbsMagnet::read_dyn_attributes(Tango::Attribute &attr,string& name) {

  magnet->readAttribute(attr,name);

}

void EbsMagnet::write_dyn_attributes(Tango::WAttribute &attr,string& name) {

  if(get_state()==Tango::MOVING)
    Tango::Except::throw_exception(
            "ErrorWrite",
            "Cannot set attribute while the magnet is cycling",
            "EbsMagnet::write_dyn_attributes");

  switch( attr.get_data_type() ) {
    case Tango::DEV_BOOLEAN: {
      bool set;
      attr.get_write_value(set);
      magnet->writeAttribute(set,name,currentDevices,psStates);
    }break;
    case Tango::DEV_DOUBLE: {
      try {
        double set;
        attr.get_write_value(set);
        if(!isnan(set)) {
          magnet->writeAttribute(set,name,currentDevices,psStates);
        } else {
          attr.rollback();
        }
      } catch (Tango::DevFailed &e) {
        string reason = e.errors[0].reason.in();
        if(reason=="Frozen") {
          // Restore setpoint without throwing if attribute is frozen
          attr.rollback();
          return;
        }
        throw e;
      }
    }break;
    default:
      Tango::Except::throw_exception(
              "ErrorWrite",
              "Unexpected type for " + name,
              "EbsMagnet::write_dyn_attributes");
  }

}

Tango::DevBoolean EbsMagnet::execute_setpointcheck_command(Tango::DevDouble strength,string& name) {

  return magnet->setpointCheck(strength,name);

}

void EbsMagnet::read_minmax() {

  if(minmaxAssigned)
    return;

  try {

    vector<string> attName;
    attName.emplace_back("Current");

    for (size_t i = 0; i < currentDevices.size(); i++) {

      Tango::DeviceProxy *ds = psGroup->get_device(i+1);
      vector<Tango::AttributeInfoEx> *conf = ds->get_attribute_config_ex(attName);
      try {
        setCurrentsMin[i] = stod((*conf)[0].min_value);
        setCurrentsMax[i] = stod((*conf)[0].max_value);
      } catch (std::invalid_argument& e) {
        cerr << "Warning, min/max not set for " << currentDevices[i] << endl;
        setCurrentsMin[i] = NAN;
        setCurrentsMax[i] = NAN;
      }
      delete conf;

    }

    // Override max setpoint for main current
    if( magnet->hasMainCurrent() )
      setCurrentsMax[0] = mainCurrentMax;

    minmaxAssigned = true;

  } catch (Tango::DevFailed& e) {}

}

void EbsMagnet::update_corrector_state() {

  // Compute corrector state

  int nbUnknown = 0;
  int nbFault = 0;
  int nbAlarm = 0;
  int nbOff = 0;
  int nbDisabled = 0;

  for(size_t i=(magnet->hasMainCurrent()?1:0);i<psStates.size();i++) {

    nbUnknown += (psStates[i]==Tango::UNKNOWN);
    nbFault+= (psStates[i]==Tango::FAULT);
    nbAlarm+= (psStates[i]==Tango::ALARM);
    nbOff+= (psStates[i]==Tango::OFF);
    nbDisabled+= (psStates[i]==Tango::DISABLE);

  }

  if (nbUnknown) {
    attr_CorrectorState_read[0] = Tango::UNKNOWN;
  } else if (nbFault) {
    attr_CorrectorState_read[0] = Tango::FAULT;
  } else if (nbAlarm) {
    attr_CorrectorState_read[0] = Tango::ALARM;
  } else if (nbDisabled) {
    attr_CorrectorState_read[0] = Tango::DISABLE;
  } else if (nbOff) {
    attr_CorrectorState_read[0] = Tango::OFF;
  } else {
    attr_CorrectorState_read[0] = Tango::ON;
  }

}

void EbsMagnet::read_ps() {

  // Read given scalar from each power supply
  vector<double> avgACCurrents;

  vector<string> attNames = {"State","Status","Current"};
  if( isSH ) {
    // FOFB SH correctors, add AC current average attribute
    attNames.emplace_back("SetCurrentAverage");
    avgACCurrents.assign(magnet->getCurrentNumber(), NAN);
  }

  // A reading should not exceed 1,5sec (polling @ 2sec)
  psGroup->set_timeout_millis(1500);
  Tango::GroupAttrReplyList arl = psGroup->read_attributes(attNames);
  psGroup->set_timeout_millis(psTimeout);
  size_t nbAtt = attNames.size();

  string error;
  for (size_t i = 0; i < arl.size(); i++) {
    size_t idx = i / nbAtt;

    if (arl[i].has_failed() || arl[i].get_data().get_quality()==Tango::ATTR_INVALID) {

      switch(i%nbAtt) {
        case 0: {
          psStates[idx] = Tango::UNKNOWN;
        } break;
        case 1: {
          if(arl[i].has_failed())
            psStatus[idx] = arl[i].get_err_stack()[0].desc.in();
          else
            psStatus[idx] = "INVALID";
        } break;
        case 2: {
          currents[idx] = NAN;
          setCurrents[idx] = NAN;
        } break;
        case 3: {
          avgACCurrents[idx] = NAN;
        } break;
      }

    } else {

      switch(i%nbAtt) {
        case 0: {
          arl[i] >> psStates[idx];
        } break;
        case 1: {
          arl[i] >> psStatus[idx];
        } break;
        case 2: {
          vector<double> values;
          arl[i]>>values;
          currents[idx] = values[0];
          setCurrents[idx] = values[1];
        } break;
        case 3: {
          arl[i] >> avgACCurrents[idx];
        } break;
      }

    }
  }

  if(isSH) {
    // Compute corresponding pseudo current of the SH power supply AC component
    magnet->computePseudoCurrentsFromCurrents(avgACCurrents, avgACPseudoCurrents);
    magnet->computePseudoCurrentsFromCurrents(currents,pseudoCurrents);
    magnet->computePseudoCurrentsFromCurrents(setCurrents,setPseudoCurrents);
  }

}
void EbsMagnet::exec_command(string cmdName) {

  Tango::GroupCmdReplyList grl = psGroup->command_inout(cmdName);
  string errMsg;
  for(size_t i=0;i<grl.size();i++) {
    if(grl[i].has_failed()) {
      errMsg += currentDevices[i] + ":" + grl[i].get_err_stack().get_buffer()[0].desc.in() + "\n";
      errMsg += "\n";
    }
  }
  if(!errMsg.empty()) {
    errMsg.pop_back();
    Tango::Except::throw_exception(
            "Error" + cmdName,
            errMsg,
            "EbsMagnet::exec_command");
  }

}

time_t EbsMagnet::get_ticks() {

  struct timeval tv;
  gettimeofday(&tv, nullptr);
  return ((tv.tv_sec) * 1000 + tv.tv_usec / 1000);

}

/*----- PROTECTED REGION END -----*/	//	EbsMagnet::namespace_ending
} //	namespace
