/*----- PROTECTED REGION ID(EbsMagnet.h) ENABLED START -----*/
//=============================================================================
//
// file :        EbsMagnet.h
//
// description : Include file for the EbsMagnet class
//
// project :     EbsMagnet
//
// This file is part of Tango device class.
// 
// Tango is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Tango is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
// 
//
// Copyright (C): 2022
//                European Synchrotron Radiation Facility
//                BP 220, Grenoble 38043
//                France
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================


#ifndef EbsMagnet_H
#define EbsMagnet_H

#include <tango.h>
#include "Magnet/Magnet.h"

#define MAGNET_RIGIDITY 20.0138495839
#define MAGNET_RIGIDITY_INV (1.0/MAGNET_RIGIDITY)

namespace EbsCorrector_ns {
  class EbsCorrector;
}

/*----- PROTECTED REGION END -----*/	//	EbsMagnet.h

/**
 *  EbsMagnet class description:
 *    Class to handle EBS magnets
 */

namespace EbsMagnet_ns
{
/*----- PROTECTED REGION ID(EbsMagnet::Additional Class Declarations) ENABLED START -----*/

//	Additional Class Declarations
class Magnet;
class DynAttribute;

/*----- PROTECTED REGION END -----*/	//	EbsMagnet::Additional Class Declarations

class EbsMagnet : public TANGO_BASE_CLASS
{

/*----- PROTECTED REGION ID(EbsMagnet::Data Members) ENABLED START -----*/


public:
  Tango::Group 	*psGroup;
  Magnet *magnet;
  vector<double> currents;
  vector<double> setCurrents;
  vector<double> setCurrentsMin;
  vector<double> setCurrentsMax;
  vector<Tango::DevState> psStates;
  vector<string> psStatus;
  bool minmaxAssigned;
  Tango::WAttribute *currentsAtt;
  vector<EbsCorrector_ns::EbsCorrector*> correctors;
  int psTimeout;

  // SH specific stuff
  bool isSH;
  vector<double> avgACPseudoCurrents;
  vector<double> pseudoCurrents;
  vector<double> setPseudoCurrents;

  void read_dyn_attributes(Tango::Attribute &attr,string& name);
  void write_dyn_attributes(Tango::WAttribute &attr,string& name);
  Tango::DevBoolean execute_setpointcheck_command(Tango::DevDouble,string& name);
  void read_minmax();
  void update_corrector_state();
  void read_ps();
  time_t get_ticks();
  void exec_command(string cmdName);

/*----- PROTECTED REGION END -----*/	//	EbsMagnet::Data Members

//	Device property data members
public:
	//	CrossTalkFactor:	CrossTalk strength scale correction factor
	Tango::DevDouble	crossTalkFactor;
	//	Model:	Magnet model
	string	model;
	//	Type:	Magnet Type (DQ,Q,S,O,SH3,SH5)
	string	type;
	//	SerialNumber:	
	string	serialNumber;
	//	Focusing:	Focusing magnet flag
	Tango::DevBoolean	focusing;
	//	CurrentDevices:	List of power supplies
	vector<string>	currentDevices;
	//	DesignStrength:	Design strength
	Tango::DevDouble	designStrength;
	//	MainCurrentMax:	Max value for the main current
	Tango::DevDouble	mainCurrentMax;

//	Attribute data members
public:
	Tango::DevString	*attr_SerialNumber_read;
	Tango::DevState	*attr_CorrectorState_read;
	Tango::DevBoolean	*attr_RequireCycling_read;
	Tango::DevDouble	*attr_Currents_read;
	Tango::DevDouble	*attr_Strengths_read;

//	Constructors and destructors
public:
	/**
	 * Constructs a newly device object.
	 *
	 *	@param cl	Class.
	 *	@param s 	Device Name
	 */
	EbsMagnet(Tango::DeviceClass *cl,string &s);
	/**
	 * Constructs a newly device object.
	 *
	 *	@param cl	Class.
	 *	@param s 	Device Name
	 */
	EbsMagnet(Tango::DeviceClass *cl,const char *s);
	/**
	 * Constructs a newly device object.
	 *
	 *	@param cl	Class.
	 *	@param s 	Device name
	 *	@param d	Device description.
	 */
	EbsMagnet(Tango::DeviceClass *cl,const char *s,const char *d);
	/**
	 * The device object destructor.
	 */
	~EbsMagnet() {delete_device();};


//	Miscellaneous methods
public:
	/*
	 *	will be called at device destruction or at init command.
	 */
	void delete_device();
	/*
	 *	Initialize the device
	 */
	virtual void init_device();
	/*
	 *	Read the device properties from database
	 */
	void get_device_property();
	/*
	 *	Always executed method before execution command method.
	 */
	virtual void always_executed_hook();


//	Attribute methods
public:
	//--------------------------------------------------------
	/*
	 *	Method      : EbsMagnet::read_attr_hardware()
	 *	Description : Hardware acquisition for attributes.
	 */
	//--------------------------------------------------------
	virtual void read_attr_hardware(vector<long> &attr_list);
	//--------------------------------------------------------
	/*
	 *	Method      : EbsMagnet::write_attr_hardware()
	 *	Description : Hardware writing for attributes.
	 */
	//--------------------------------------------------------
	virtual void write_attr_hardware(vector<long> &attr_list);

/**
 *	Attribute SerialNumber related methods
 *	Description: 
 *
 *	Data type:	Tango::DevString
 *	Attr type:	Scalar
 */
	virtual void read_SerialNumber(Tango::Attribute &attr);
	virtual bool is_SerialNumber_allowed(Tango::AttReqType type);
/**
 *	Attribute CorrectorState related methods
 *	Description: 
 *
 *	Data type:	Tango::DevState
 *	Attr type:	Scalar
 */
	virtual void read_CorrectorState(Tango::Attribute &attr);
	virtual bool is_CorrectorState_allowed(Tango::AttReqType type);
/**
 *	Attribute RequireCycling related methods
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
	virtual void read_RequireCycling(Tango::Attribute &attr);
	virtual bool is_RequireCycling_allowed(Tango::AttReqType type);
/**
 *	Attribute Currents related methods
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Spectrum max = 5
 */
	virtual void read_Currents(Tango::Attribute &attr);
	virtual void write_Currents(Tango::WAttribute &attr);
	virtual bool is_Currents_allowed(Tango::AttReqType type);
/**
 *	Attribute Strengths related methods
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Spectrum max = 5
 */
	virtual void read_Strengths(Tango::Attribute &attr);
	virtual void write_Strengths(Tango::WAttribute &attr);
	virtual bool is_Strengths_allowed(Tango::AttReqType type);


	//--------------------------------------------------------
	/**
	 *	Method      : EbsMagnet::add_dynamic_attributes()
	 *	Description : Add dynamic attributes if any.
	 */
	//--------------------------------------------------------
	void add_dynamic_attributes();




//	Command related methods
public:
	/**
	 *	Command State related method
	 *	Description: This command gets the device state (stored in its device_state data member) and returns it to the caller.
	 *
	 *	@returns Device state
	 */
	virtual Tango::DevState dev_state();
	/**
	 *	Command Reset related method
	 *	Description: Reset underlying power supplies
	 *
	 */
	virtual void reset();
	virtual bool is_Reset_allowed(const CORBA::Any &any);
	/**
	 *	Command SetDesignStrength related method
	 *	Description: Set the design strength (Expert usage)
	 *               (DesignStrength has to be a read only attribute)
	 *
	 *	@param argin Design strength
	 */
	virtual void set_design_strength(Tango::DevDouble argin);
	virtual bool is_SetDesignStrength_allowed(const CORBA::Any &any);
	/**
	 *	Command ApplyDeltaStrength related method
	 *	Description: Apply delta strength
	 *
	 *	@param argin Delta strength to be applied
	 */
	virtual void apply_delta_strength(Tango::DevDouble argin);
	virtual bool is_ApplyDeltaStrength_allowed(const CORBA::Any &any);
	/**
	 *	Command ResetResonanceStrength related method
	 *	Description: Set resonnance strength to 0
	 *
	 */
	virtual void reset_resonance_strength();
	virtual bool is_ResetResonanceStrength_allowed(const CORBA::Any &any);
	/**
	 *	Command MainCurrentSetpointCheck related method
	 *	Description: Check if current setpoint of the main power supply is valid.
	 *               If the magnet hasn`t a main power supply, returns true.
	 *
	 *	@param argin New main current
	 *	@returns True if PS ia able to generate the required current
	 */
	virtual Tango::DevBoolean main_current_setpoint_check(Tango::DevDouble argin);
	virtual bool is_MainCurrentSetpointCheck_allowed(const CORBA::Any &any);
	/**
	 *	Command On related method
	 *	Description: Switch ON underlying power supply
	 *
	 */
	virtual void on();
	virtual bool is_On_allowed(const CORBA::Any &any);
	/**
	 *	Command Off related method
	 *	Description: Switch OFF underlying power supply
	 *
	 */
	virtual void off();
	virtual bool is_Off_allowed(const CORBA::Any &any);
	/**
	 *	Command GetACPSCurrent related method
	 *	Description: Get the SH AC pseudo current (0=>h 1=>v 2=>sq)
	 *               If the magnet is not a SH, returns NAN
	 *
	 *	@param argin pseudo current index (0=>h 1=>v 2=>sq)
	 *	@returns Pseudo current
	 */
	virtual Tango::DevDouble get_acpscurrent(Tango::DevShort argin);
	virtual bool is_GetACPSCurrent_allowed(const CORBA::Any &any);
	/**
	 *	Command GetPSCurrent related method
	 *	Description: Get the SH pseudo current (0=>h 1=>v 2=>sq)
	 *               If the magnet is not a SH, returns NAN
	 *
	 *	@param argin Pseudo current index (0=>h 1=>v 2=>sq)
	 *	@returns Pseudo current (read,set)
	 */
	virtual Tango::DevVarDoubleArray *get_pscurrent(Tango::DevShort argin);
	virtual bool is_GetPSCurrent_allowed(const CORBA::Any &any);
	/**
	 *	Command WritePSCurrent related method
	 *	Description: Write SH pseudo current.
	 *               argin[0] = idx (0=>h 1=>v 2=>sq)
	 *               argin[1] = ps current
	 *               If the magnet is not a SH, the command has no effect.
	 *
	 *	@param argin [0]=Pseuco current idx,[1]=Pseudo current
	 */
	virtual void write_pscurrent(const Tango::DevVarDoubleArray *argin);
	virtual bool is_WritePSCurrent_allowed(const CORBA::Any &any);
	/**
	 *	Command Cycle related method
	 *	Description: Cycle the magnet
	 *
	 */
	virtual void cycle();
	virtual bool is_Cycle_allowed(const CORBA::Any &any);
	/**
	 *	Command Swap related method
	 *	Description: Send SWAP to HSM
	 *
	 */
	virtual void swap();
	virtual bool is_Swap_allowed(const CORBA::Any &any);
	/**
	 *	Command Unswap related method
	 *	Description: Send UNSWAP to HSM
	 *
	 */
	virtual void unswap();
	virtual bool is_Unswap_allowed(const CORBA::Any &any);


	//--------------------------------------------------------
	/**
	 *	Method      : EbsMagnet::add_dynamic_commands()
	 *	Description : Add dynamic commands if any.
	 */
	//--------------------------------------------------------
	void add_dynamic_commands();

/*----- PROTECTED REGION ID(EbsMagnet::Additional Method prototypes) ENABLED START -----*/

//	Additional Method prototypes

/*----- PROTECTED REGION END -----*/	//	EbsMagnet::Additional Method prototypes
};

/*----- PROTECTED REGION ID(EbsMagnet::Additional Classes Definitions) ENABLED START -----*/

//	Additional Classes Definitions

/*----- PROTECTED REGION END -----*/	//	EbsMagnet::Additional Classes Definitions

}	//	End of namespace

#endif   //	EbsMagnet_H
