/*----- PROTECTED REGION ID(EbsMagnetClass.cpp) ENABLED START -----*/
//=============================================================================
//
// file :        EbsMagnetClass.cpp
//
// description : C++ source for the EbsMagnetClass.
//               A singleton class derived from DeviceClass.
//               It implements the command and attribute list
//               and all properties and methods required
//               by the EbsMagnet once per process.
//
// project :     EbsMagnet
//
// This file is part of Tango device class.
// 
// Tango is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Tango is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
// 
//
// Copyright (C): 2022
//                European Synchrotron Radiation Facility
//                BP 220, Grenoble 38043
//                France
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================


#include <EbsMagnetClass.h>

/*----- PROTECTED REGION END -----*/	//	EbsMagnetClass.cpp

//-------------------------------------------------------------------
/**
 *	Create EbsMagnetClass singleton and
 *	return it in a C function for Python usage
 */
//-------------------------------------------------------------------
extern "C" {
#ifdef _TG_WINDOWS_

__declspec(dllexport)

#endif

	Tango::DeviceClass *_create_EbsMagnet_class(const char *name) {
		return EbsMagnet_ns::EbsMagnetClass::init(name);
	}
}

namespace EbsMagnet_ns
{
//===================================================================
//	Initialize pointer for singleton pattern
//===================================================================
EbsMagnetClass *EbsMagnetClass::_instance = NULL;

//--------------------------------------------------------
/**
 * method : 		EbsMagnetClass::EbsMagnetClass(string &s)
 * description : 	constructor for the EbsMagnetClass
 *
 * @param s	The class name
 */
//--------------------------------------------------------
EbsMagnetClass::EbsMagnetClass(string &s):Tango::DeviceClass(s)
{
	cout2 << "Entering EbsMagnetClass constructor" << endl;
	set_default_property();
	write_class_property();

	/*----- PROTECTED REGION ID(EbsMagnetClass::constructor) ENABLED START -----*/
	
	/*----- PROTECTED REGION END -----*/	//	EbsMagnetClass::constructor

	cout2 << "Leaving EbsMagnetClass constructor" << endl;
}

//--------------------------------------------------------
/**
 * method : 		EbsMagnetClass::~EbsMagnetClass()
 * description : 	destructor for the EbsMagnetClass
 */
//--------------------------------------------------------
EbsMagnetClass::~EbsMagnetClass()
{
	/*----- PROTECTED REGION ID(EbsMagnetClass::destructor) ENABLED START -----*/
	
	/*----- PROTECTED REGION END -----*/	//	EbsMagnetClass::destructor

	_instance = NULL;
}


//--------------------------------------------------------
/**
 * method : 		EbsMagnetClass::init
 * description : 	Create the object if not already done.
 *                  Otherwise, just return a pointer to the object
 *
 * @param	name	The class name
 */
//--------------------------------------------------------
EbsMagnetClass *EbsMagnetClass::init(const char *name)
{
	if (_instance == NULL)
	{
		try
		{
			string s(name);
			_instance = new EbsMagnetClass(s);
		}
		catch (bad_alloc &)
		{
			throw;
		}
	}
	return _instance;
}

//--------------------------------------------------------
/**
 * method : 		EbsMagnetClass::instance
 * description : 	Check if object already created,
 *                  and return a pointer to the object
 */
//--------------------------------------------------------
EbsMagnetClass *EbsMagnetClass::instance()
{
	if (_instance == NULL)
	{
		cerr << "Class is not initialised !!" << endl;
		exit(-1);
	}
	return _instance;
}



//===================================================================
//	Command execution method calls
//===================================================================
//--------------------------------------------------------
/**
 * method : 		ResetClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *ResetClass::execute(Tango::DeviceImpl *device, TANGO_UNUSED(const CORBA::Any &in_any))
{
	cout2 << "ResetClass::execute(): arrived" << endl;
	((static_cast<EbsMagnet *>(device))->reset());
	return new CORBA::Any();
}

//--------------------------------------------------------
/**
 * method : 		SetDesignStrengthClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *SetDesignStrengthClass::execute(Tango::DeviceImpl *device, const CORBA::Any &in_any)
{
	cout2 << "SetDesignStrengthClass::execute(): arrived" << endl;
	Tango::DevDouble argin;
	extract(in_any, argin);
	((static_cast<EbsMagnet *>(device))->set_design_strength(argin));
	return new CORBA::Any();
}

//--------------------------------------------------------
/**
 * method : 		ApplyDeltaStrengthClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *ApplyDeltaStrengthClass::execute(Tango::DeviceImpl *device, const CORBA::Any &in_any)
{
	cout2 << "ApplyDeltaStrengthClass::execute(): arrived" << endl;
	Tango::DevDouble argin;
	extract(in_any, argin);
	((static_cast<EbsMagnet *>(device))->apply_delta_strength(argin));
	return new CORBA::Any();
}

//--------------------------------------------------------
/**
 * method : 		ResetResonanceStrengthClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *ResetResonanceStrengthClass::execute(Tango::DeviceImpl *device, TANGO_UNUSED(const CORBA::Any &in_any))
{
	cout2 << "ResetResonanceStrengthClass::execute(): arrived" << endl;
	((static_cast<EbsMagnet *>(device))->reset_resonance_strength());
	return new CORBA::Any();
}

//--------------------------------------------------------
/**
 * method : 		MainCurrentSetpointCheckClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *MainCurrentSetpointCheckClass::execute(Tango::DeviceImpl *device, const CORBA::Any &in_any)
{
	cout2 << "MainCurrentSetpointCheckClass::execute(): arrived" << endl;
	Tango::DevDouble argin;
	extract(in_any, argin);
	return insert((static_cast<EbsMagnet *>(device))->main_current_setpoint_check(argin));
}

//--------------------------------------------------------
/**
 * method : 		OnClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *OnClass::execute(Tango::DeviceImpl *device, TANGO_UNUSED(const CORBA::Any &in_any))
{
	cout2 << "OnClass::execute(): arrived" << endl;
	((static_cast<EbsMagnet *>(device))->on());
	return new CORBA::Any();
}

//--------------------------------------------------------
/**
 * method : 		OffClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *OffClass::execute(Tango::DeviceImpl *device, TANGO_UNUSED(const CORBA::Any &in_any))
{
	cout2 << "OffClass::execute(): arrived" << endl;
	((static_cast<EbsMagnet *>(device))->off());
	return new CORBA::Any();
}

//--------------------------------------------------------
/**
 * method : 		GetACPSCurrentClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *GetACPSCurrentClass::execute(Tango::DeviceImpl *device, const CORBA::Any &in_any)
{
	cout2 << "GetACPSCurrentClass::execute(): arrived" << endl;
	Tango::DevShort argin;
	extract(in_any, argin);
	return insert((static_cast<EbsMagnet *>(device))->get_acpscurrent(argin));
}

//--------------------------------------------------------
/**
 * method : 		GetPSCurrentClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *GetPSCurrentClass::execute(Tango::DeviceImpl *device, const CORBA::Any &in_any)
{
	cout2 << "GetPSCurrentClass::execute(): arrived" << endl;
	Tango::DevShort argin;
	extract(in_any, argin);
	return insert((static_cast<EbsMagnet *>(device))->get_pscurrent(argin));
}

//--------------------------------------------------------
/**
 * method : 		WritePSCurrentClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *WritePSCurrentClass::execute(Tango::DeviceImpl *device, const CORBA::Any &in_any)
{
	cout2 << "WritePSCurrentClass::execute(): arrived" << endl;
	const Tango::DevVarDoubleArray *argin;
	extract(in_any, argin);
	((static_cast<EbsMagnet *>(device))->write_pscurrent(argin));
	return new CORBA::Any();
}

//--------------------------------------------------------
/**
 * method : 		CycleClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *CycleClass::execute(Tango::DeviceImpl *device, TANGO_UNUSED(const CORBA::Any &in_any))
{
	cout2 << "CycleClass::execute(): arrived" << endl;
	((static_cast<EbsMagnet *>(device))->cycle());
	return new CORBA::Any();
}

//--------------------------------------------------------
/**
 * method : 		SwapClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *SwapClass::execute(Tango::DeviceImpl *device, TANGO_UNUSED(const CORBA::Any &in_any))
{
	cout2 << "SwapClass::execute(): arrived" << endl;
	((static_cast<EbsMagnet *>(device))->swap());
	return new CORBA::Any();
}

//--------------------------------------------------------
/**
 * method : 		UnswapClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *UnswapClass::execute(Tango::DeviceImpl *device, TANGO_UNUSED(const CORBA::Any &in_any))
{
	cout2 << "UnswapClass::execute(): arrived" << endl;
	((static_cast<EbsMagnet *>(device))->unswap());
	return new CORBA::Any();
}


//===================================================================
//	Properties management
//===================================================================
//--------------------------------------------------------
/**
 *	Method      : EbsMagnetClass::get_class_property()
 *	Description : Get the class property for specified name.
 */
//--------------------------------------------------------
Tango::DbDatum EbsMagnetClass::get_class_property(string &prop_name)
{
	for (unsigned int i=0 ; i<cl_prop.size() ; i++)
		if (cl_prop[i].name == prop_name)
			return cl_prop[i];
	//	if not found, returns  an empty DbDatum
	return Tango::DbDatum(prop_name);
}

//--------------------------------------------------------
/**
 *	Method      : EbsMagnetClass::get_default_device_property()
 *	Description : Return the default value for device property.
 */
//--------------------------------------------------------
Tango::DbDatum EbsMagnetClass::get_default_device_property(string &prop_name)
{
	for (unsigned int i=0 ; i<dev_def_prop.size() ; i++)
		if (dev_def_prop[i].name == prop_name)
			return dev_def_prop[i];
	//	if not found, return  an empty DbDatum
	return Tango::DbDatum(prop_name);
}

//--------------------------------------------------------
/**
 *	Method      : EbsMagnetClass::get_default_class_property()
 *	Description : Return the default value for class property.
 */
//--------------------------------------------------------
Tango::DbDatum EbsMagnetClass::get_default_class_property(string &prop_name)
{
	for (unsigned int i=0 ; i<cl_def_prop.size() ; i++)
		if (cl_def_prop[i].name == prop_name)
			return cl_def_prop[i];
	//	if not found, return  an empty DbDatum
	return Tango::DbDatum(prop_name);
}


//--------------------------------------------------------
/**
 *	Method      : EbsMagnetClass::set_default_property()
 *	Description : Set default property (class and device) for wizard.
 *                For each property, add to wizard property name and description.
 *                If default value has been set, add it to wizard property and
 *                store it in a DbDatum.
 */
//--------------------------------------------------------
void EbsMagnetClass::set_default_property()
{
	string	prop_name;
	string	prop_desc;
	string	prop_def;
	vector<string>	vect_data;

	//	Set Default Class Properties

	//	Set Default device Properties
	prop_name = "CrossTalkFactor";
	prop_desc = "CrossTalk strength scale correction factor";
	prop_def  = "1.0";
	vect_data.clear();
	vect_data.push_back("1.0");
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "Model";
	prop_desc = "Magnet model";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "Type";
	prop_desc = "Magnet Type (DQ,Q,S,O,SH3,SH5)";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "SerialNumber";
	prop_desc = "";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "Focusing";
	prop_desc = "Focusing magnet flag";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "CurrentDevices";
	prop_desc = "List of power supplies";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "DesignStrength";
	prop_desc = "Design strength";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "MainCurrentMax";
	prop_desc = "Max value for the main current";
	prop_def  = "110";
	vect_data.clear();
	vect_data.push_back("110");
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
}

//--------------------------------------------------------
/**
 *	Method      : EbsMagnetClass::write_class_property()
 *	Description : Set class description fields as property in database
 */
//--------------------------------------------------------
void EbsMagnetClass::write_class_property()
{
	//	First time, check if database used
	if (Tango::Util::_UseDb == false)
		return;

	Tango::DbData	data;
	string	classname = get_name();
	string	header;
	string::size_type	start, end;

	//	Put title
	Tango::DbDatum	title("ProjectTitle");
	string	str_title("EbsMagnet");
	title << str_title;
	data.push_back(title);

	//	Put Description
	Tango::DbDatum	description("Description");
	vector<string>	str_desc;
	str_desc.push_back("Class to handle EBS magnets");
	description << str_desc;
	data.push_back(description);

	//  Put inheritance
	Tango::DbDatum	inher_datum("InheritedFrom");
	vector<string> inheritance;
	inheritance.push_back("TANGO_BASE_CLASS");
	inher_datum << inheritance;
	data.push_back(inher_datum);

	//	Call database and and values
	get_db_class()->put_property(data);
}

//===================================================================
//	Factory methods
//===================================================================

//--------------------------------------------------------
/**
 *	Method      : EbsMagnetClass::device_factory()
 *	Description : Create the device object(s)
 *                and store them in the device list
 */
//--------------------------------------------------------
void EbsMagnetClass::device_factory(const Tango::DevVarStringArray *devlist_ptr)
{
	/*----- PROTECTED REGION ID(EbsMagnetClass::device_factory_before) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	EbsMagnetClass::device_factory_before

	//	Create devices and add it into the device list
	for (unsigned long i=0 ; i<devlist_ptr->length() ; i++)
	{
		cout4 << "Device name : " << (*devlist_ptr)[i].in() << endl;
		device_list.push_back(new EbsMagnet(this, (*devlist_ptr)[i]));
	}

	//	Manage dynamic attributes if any
	erase_dynamic_attributes(devlist_ptr, get_class_attr()->get_attr_list());

	//	Export devices to the outside world
	for (unsigned long i=1 ; i<=devlist_ptr->length() ; i++)
	{
		//	Add dynamic attributes if any
		EbsMagnet *dev = static_cast<EbsMagnet *>(device_list[device_list.size()-i]);
		dev->add_dynamic_attributes();

		//	Check before if database used.
		if ((Tango::Util::_UseDb == true) && (Tango::Util::_FileDb == false))
			export_device(dev);
		else
			export_device(dev, dev->get_name().c_str());
	}

	/*----- PROTECTED REGION ID(EbsMagnetClass::device_factory_after) ENABLED START -----*/

  //	Add dynamic commands
  for (unsigned long i=1 ; i<=devlist_ptr->length() ; i++) {
    auto *dev = dynamic_cast<EbsMagnet *>(device_list[device_list.size()-i]);
    dev->add_dynamic_commands();
  }

	/*----- PROTECTED REGION END -----*/	//	EbsMagnetClass::device_factory_after
}
//--------------------------------------------------------
/**
 *	Method      : EbsMagnetClass::attribute_factory()
 *	Description : Create the attribute object(s)
 *                and store them in the attribute list
 */
//--------------------------------------------------------
void EbsMagnetClass::attribute_factory(vector<Tango::Attr *> &att_list)
{
	/*----- PROTECTED REGION ID(EbsMagnetClass::attribute_factory_before) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	EbsMagnetClass::attribute_factory_before
	//	Attribute : SerialNumber
	SerialNumberAttrib	*serialnumber = new SerialNumberAttrib();
	Tango::UserDefaultAttrProp	serialnumber_prop;
	//	description	not set for SerialNumber
	//	label	not set for SerialNumber
	//	unit	not set for SerialNumber
	//	standard_unit	not set for SerialNumber
	//	display_unit	not set for SerialNumber
	//	format	not set for SerialNumber
	//	max_value	not set for SerialNumber
	//	min_value	not set for SerialNumber
	//	max_alarm	not set for SerialNumber
	//	min_alarm	not set for SerialNumber
	//	max_warning	not set for SerialNumber
	//	min_warning	not set for SerialNumber
	//	delta_t	not set for SerialNumber
	//	delta_val	not set for SerialNumber
	
	serialnumber->set_default_properties(serialnumber_prop);
	//	Not Polled
	serialnumber->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(serialnumber);

	//	Attribute : CorrectorState
	CorrectorStateAttrib	*correctorstate = new CorrectorStateAttrib();
	Tango::UserDefaultAttrProp	correctorstate_prop;
	//	description	not set for CorrectorState
	//	label	not set for CorrectorState
	//	unit	not set for CorrectorState
	//	standard_unit	not set for CorrectorState
	//	display_unit	not set for CorrectorState
	//	format	not set for CorrectorState
	//	max_value	not set for CorrectorState
	//	min_value	not set for CorrectorState
	//	max_alarm	not set for CorrectorState
	//	min_alarm	not set for CorrectorState
	//	max_warning	not set for CorrectorState
	//	min_warning	not set for CorrectorState
	//	delta_t	not set for CorrectorState
	//	delta_val	not set for CorrectorState
	
	correctorstate->set_default_properties(correctorstate_prop);
	//	Not Polled
	correctorstate->set_disp_level(Tango::EXPERT);
	//	Not Memorized
	att_list.push_back(correctorstate);

	//	Attribute : RequireCycling
	RequireCyclingAttrib	*requirecycling = new RequireCyclingAttrib();
	Tango::UserDefaultAttrProp	requirecycling_prop;
	//	description	not set for RequireCycling
	//	label	not set for RequireCycling
	//	unit	not set for RequireCycling
	//	standard_unit	not set for RequireCycling
	//	display_unit	not set for RequireCycling
	//	format	not set for RequireCycling
	//	max_value	not set for RequireCycling
	//	min_value	not set for RequireCycling
	//	max_alarm	not set for RequireCycling
	//	min_alarm	not set for RequireCycling
	//	max_warning	not set for RequireCycling
	//	min_warning	not set for RequireCycling
	//	delta_t	not set for RequireCycling
	//	delta_val	not set for RequireCycling
	
	requirecycling->set_default_properties(requirecycling_prop);
	//	Not Polled
	requirecycling->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(requirecycling);

	//	Attribute : Currents
	CurrentsAttrib	*currents = new CurrentsAttrib();
	Tango::UserDefaultAttrProp	currents_prop;
	//	description	not set for Currents
	//	label	not set for Currents
	currents_prop.set_unit("A");
	//	standard_unit	not set for Currents
	//	display_unit	not set for Currents
	currents_prop.set_format("%9.6f");
	//	max_value	not set for Currents
	//	min_value	not set for Currents
	//	max_alarm	not set for Currents
	//	min_alarm	not set for Currents
	//	max_warning	not set for Currents
	//	min_warning	not set for Currents
	//	delta_t	not set for Currents
	//	delta_val	not set for Currents
	
	currents->set_default_properties(currents_prop);
	//	Not Polled
	currents->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(currents);

	//	Attribute : Strengths
	StrengthsAttrib	*strengths = new StrengthsAttrib();
	Tango::UserDefaultAttrProp	strengths_prop;
	//	description	not set for Strengths
	//	label	not set for Strengths
	//	unit	not set for Strengths
	//	standard_unit	not set for Strengths
	//	display_unit	not set for Strengths
	//	format	not set for Strengths
	//	max_value	not set for Strengths
	//	min_value	not set for Strengths
	//	max_alarm	not set for Strengths
	//	min_alarm	not set for Strengths
	//	max_warning	not set for Strengths
	//	min_warning	not set for Strengths
	//	delta_t	not set for Strengths
	//	delta_val	not set for Strengths
	
	strengths->set_default_properties(strengths_prop);
	//	Not Polled
	strengths->set_disp_level(Tango::EXPERT);
	//	Not Memorized
	att_list.push_back(strengths);


	//	Create a list of static attributes
	create_static_attribute_list(get_class_attr()->get_attr_list());
	/*----- PROTECTED REGION ID(EbsMagnetClass::attribute_factory_after) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	EbsMagnetClass::attribute_factory_after
}
//--------------------------------------------------------
/**
 *	Method      : EbsMagnetClass::pipe_factory()
 *	Description : Create the pipe object(s)
 *                and store them in the pipe list
 */
//--------------------------------------------------------
void EbsMagnetClass::pipe_factory()
{
	/*----- PROTECTED REGION ID(EbsMagnetClass::pipe_factory_before) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	EbsMagnetClass::pipe_factory_before
	/*----- PROTECTED REGION ID(EbsMagnetClass::pipe_factory_after) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	EbsMagnetClass::pipe_factory_after
}
//--------------------------------------------------------
/**
 *	Method      : EbsMagnetClass::command_factory()
 *	Description : Create the command object(s)
 *                and store them in the command list
 */
//--------------------------------------------------------
void EbsMagnetClass::command_factory()
{
	/*----- PROTECTED REGION ID(EbsMagnetClass::command_factory_before) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	EbsMagnetClass::command_factory_before

	//	Set polling perod for command State
	Tango::Command	&stateCmd = get_cmd_by_name("State");
	stateCmd.set_polling_period(2000);
	

	//	Command Reset
	ResetClass	*pResetCmd =
		new ResetClass("Reset",
			Tango::DEV_VOID, Tango::DEV_VOID,
			"",
			"",
			Tango::OPERATOR);
	command_list.push_back(pResetCmd);

	//	Command SetDesignStrength
	SetDesignStrengthClass	*pSetDesignStrengthCmd =
		new SetDesignStrengthClass("SetDesignStrength",
			Tango::DEV_DOUBLE, Tango::DEV_VOID,
			"Design strength",
			"",
			Tango::EXPERT);
	command_list.push_back(pSetDesignStrengthCmd);

	//	Command ApplyDeltaStrength
	ApplyDeltaStrengthClass	*pApplyDeltaStrengthCmd =
		new ApplyDeltaStrengthClass("ApplyDeltaStrength",
			Tango::DEV_DOUBLE, Tango::DEV_VOID,
			"Delta strength to be applied",
			"",
			Tango::EXPERT);
	command_list.push_back(pApplyDeltaStrengthCmd);

	//	Command ResetResonanceStrength
	ResetResonanceStrengthClass	*pResetResonanceStrengthCmd =
		new ResetResonanceStrengthClass("ResetResonanceStrength",
			Tango::DEV_VOID, Tango::DEV_VOID,
			"",
			"",
			Tango::EXPERT);
	command_list.push_back(pResetResonanceStrengthCmd);

	//	Command MainCurrentSetpointCheck
	MainCurrentSetpointCheckClass	*pMainCurrentSetpointCheckCmd =
		new MainCurrentSetpointCheckClass("MainCurrentSetpointCheck",
			Tango::DEV_DOUBLE, Tango::DEV_BOOLEAN,
			"New main current",
			"True if PS ia able to generate the required current",
			Tango::EXPERT);
	command_list.push_back(pMainCurrentSetpointCheckCmd);

	//	Command On
	OnClass	*pOnCmd =
		new OnClass("On",
			Tango::DEV_VOID, Tango::DEV_VOID,
			"",
			"",
			Tango::OPERATOR);
	command_list.push_back(pOnCmd);

	//	Command Off
	OffClass	*pOffCmd =
		new OffClass("Off",
			Tango::DEV_VOID, Tango::DEV_VOID,
			"",
			"",
			Tango::OPERATOR);
	command_list.push_back(pOffCmd);

	//	Command GetACPSCurrent
	GetACPSCurrentClass	*pGetACPSCurrentCmd =
		new GetACPSCurrentClass("GetACPSCurrent",
			Tango::DEV_SHORT, Tango::DEV_DOUBLE,
			"pseudo current index (0=>h 1=>v 2=>sq)",
			"Pseudo current",
			Tango::EXPERT);
	command_list.push_back(pGetACPSCurrentCmd);

	//	Command GetPSCurrent
	GetPSCurrentClass	*pGetPSCurrentCmd =
		new GetPSCurrentClass("GetPSCurrent",
			Tango::DEV_SHORT, Tango::DEVVAR_DOUBLEARRAY,
			"Pseudo current index (0=>h 1=>v 2=>sq)",
			"Pseudo current (read,set)",
			Tango::EXPERT);
	command_list.push_back(pGetPSCurrentCmd);

	//	Command WritePSCurrent
	WritePSCurrentClass	*pWritePSCurrentCmd =
		new WritePSCurrentClass("WritePSCurrent",
			Tango::DEVVAR_DOUBLEARRAY, Tango::DEV_VOID,
			"[0]=Pseuco current idx,[1]=Pseudo current",
			"",
			Tango::EXPERT);
	command_list.push_back(pWritePSCurrentCmd);

	//	Command Cycle
	CycleClass	*pCycleCmd =
		new CycleClass("Cycle",
			Tango::DEV_VOID, Tango::DEV_VOID,
			"",
			"",
			Tango::OPERATOR);
	command_list.push_back(pCycleCmd);

	//	Command Swap
	SwapClass	*pSwapCmd =
		new SwapClass("Swap",
			Tango::DEV_VOID, Tango::DEV_VOID,
			"",
			"",
			Tango::OPERATOR);
	command_list.push_back(pSwapCmd);

	//	Command Unswap
	UnswapClass	*pUnswapCmd =
		new UnswapClass("Unswap",
			Tango::DEV_VOID, Tango::DEV_VOID,
			"",
			"",
			Tango::OPERATOR);
	command_list.push_back(pUnswapCmd);

	/*----- PROTECTED REGION ID(EbsMagnetClass::command_factory_after) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	EbsMagnetClass::command_factory_after
}

//===================================================================
//	Dynamic attributes related methods
//===================================================================

//--------------------------------------------------------
/**
 * method : 		EbsMagnetClass::create_static_attribute_list
 * description : 	Create the a list of static attributes
 *
 * @param	att_list	the ceated attribute list
 */
//--------------------------------------------------------
void EbsMagnetClass::create_static_attribute_list(vector<Tango::Attr *> &att_list)
{
	for (unsigned long i=0 ; i<att_list.size() ; i++)
	{
		string att_name(att_list[i]->get_name());
		transform(att_name.begin(), att_name.end(), att_name.begin(), ::tolower);
		defaultAttList.push_back(att_name);
	}

	cout2 << defaultAttList.size() << " attributes in default list" << endl;

	/*----- PROTECTED REGION ID(EbsMagnetClass::create_static_att_list) ENABLED START -----*/
	
	/*----- PROTECTED REGION END -----*/	//	EbsMagnetClass::create_static_att_list
}


//--------------------------------------------------------
/**
 * method : 		EbsMagnetClass::erase_dynamic_attributes
 * description : 	delete the dynamic attributes if any.
 *
 * @param	devlist_ptr	the device list pointer
 * @param	list of all attributes
 */
//--------------------------------------------------------
void EbsMagnetClass::erase_dynamic_attributes(const Tango::DevVarStringArray *devlist_ptr, vector<Tango::Attr *> &att_list)
{
	Tango::Util *tg = Tango::Util::instance();

	for (unsigned long i=0 ; i<devlist_ptr->length() ; i++)
	{
		Tango::DeviceImpl *dev_impl = tg->get_device_by_name(((string)(*devlist_ptr)[i]).c_str());
		EbsMagnet *dev = static_cast<EbsMagnet *> (dev_impl);

		vector<Tango::Attribute *> &dev_att_list = dev->get_device_attr()->get_attribute_list();
		vector<Tango::Attribute *>::iterator ite_att;
		for (ite_att=dev_att_list.begin() ; ite_att != dev_att_list.end() ; ++ite_att)
		{
			string att_name((*ite_att)->get_name_lower());
			if ((att_name == "state") || (att_name == "status"))
				continue;
			vector<string>::iterator ite_str = find(defaultAttList.begin(), defaultAttList.end(), att_name);
			if (ite_str == defaultAttList.end())
			{
				cout2 << att_name << " is a UNWANTED dynamic attribute for device " << (*devlist_ptr)[i] << endl;
				Tango::Attribute &att = dev->get_device_attr()->get_attr_by_name(att_name.c_str());
				dev->remove_attribute(att_list[att.get_attr_idx()], true, false);
				--ite_att;
			}
		}
	}
	/*----- PROTECTED REGION ID(EbsMagnetClass::erase_dynamic_attributes) ENABLED START -----*/
	
	/*----- PROTECTED REGION END -----*/	//	EbsMagnetClass::erase_dynamic_attributes
}

//--------------------------------------------------------
/**
 *	Method      : EbsMagnetClass::get_attr_object_by_name()
 *	Description : returns Tango::Attr * object found by name
 */
//--------------------------------------------------------
Tango::Attr *EbsMagnetClass::get_attr_object_by_name(vector<Tango::Attr *> &att_list, string attname)
{
	vector<Tango::Attr *>::iterator it;
	for (it=att_list.begin() ; it<att_list.end() ; ++it)
		if ((*it)->get_name()==attname)
			return (*it);
	//	Attr does not exist
	return NULL;
}


/*----- PROTECTED REGION ID(EbsMagnetClass::Additional Methods) ENABLED START -----*/

/*----- PROTECTED REGION END -----*/	//	EbsMagnetClass::Additional Methods
} //	namespace
