/*----- PROTECTED REGION ID(EbsMagnetClass.h) ENABLED START -----*/
//=============================================================================
//
// file :        EbsMagnetClass.h
//
// description : Include for the EbsMagnet root class.
//               This class is the singleton class for
//                the EbsMagnet device class.
//               It contains all properties and methods which the 
//               EbsMagnet requires only once e.g. the commands.
//
// project :     EbsMagnet
//
// This file is part of Tango device class.
// 
// Tango is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Tango is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
// 
//
// Copyright (C): 2022
//                European Synchrotron Radiation Facility
//                BP 220, Grenoble 38043
//                France
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================


#ifndef EbsMagnetClass_H
#define EbsMagnetClass_H

#include <tango.h>
#include <EbsMagnet.h>


/*----- PROTECTED REGION END -----*/	//	EbsMagnetClass.h


namespace EbsMagnet_ns
{
/*----- PROTECTED REGION ID(EbsMagnetClass::classes for dynamic creation) ENABLED START -----*/

// Dynamic scalar attribute
class DynAttribute: public Tango::Attr {

public:

  //-----------------------------------------------------------------------------
  // Constructor

  DynAttribute(const char *attName,long data_type,Tango::AttrWriteType rwType)
          :Attr(attName,data_type,rwType) {};

  //-----------------------------------------------------------------------------
  // Write/Read mehod

  virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att) {
    (static_cast<EbsMagnet *>(dev))->read_dyn_attributes(att,this->name);}
  virtual void write(Tango::DeviceImpl *dev,Tango::WAttribute &att) {
    (static_cast<EbsMagnet *>(dev))->write_dyn_attributes(att,this->name);}

};

// Dynamic command
class SetpointCheckCommand : public Tango::Command
{
public:
  SetpointCheckCommand(const char   *name, Tango::DispLevel  level)
          :Command(name, Tango::DEV_DOUBLE , Tango::DEV_BOOLEAN ,"Desired strength",
                   "True if the underlying power supplies can deliver the required current for the required strength",
                   level)	{};

  ~SetpointCheckCommand() {};

  virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &in_any) {
    Tango::DevDouble argin;
    extract(in_any, argin);
    return insert((static_cast<EbsMagnet *>(dev))->execute_setpointcheck_command(argin,this->name));
  }
  virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
  {return true;}
};

/*----- PROTECTED REGION END -----*/	//	EbsMagnetClass::classes for dynamic creation

//=========================================
//	Define classes for attributes
//=========================================
//	Attribute SerialNumber class definition
class SerialNumberAttrib: public Tango::Attr
{
public:
	SerialNumberAttrib():Attr("SerialNumber",
			Tango::DEV_STRING, Tango::READ) {};
	~SerialNumberAttrib() {};
	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
		{(static_cast<EbsMagnet *>(dev))->read_SerialNumber(att);}
	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
		{return (static_cast<EbsMagnet *>(dev))->is_SerialNumber_allowed(ty);}
};

//	Attribute CorrectorState class definition
class CorrectorStateAttrib: public Tango::Attr
{
public:
	CorrectorStateAttrib():Attr("CorrectorState",
			Tango::DEV_STATE, Tango::READ) {};
	~CorrectorStateAttrib() {};
	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
		{(static_cast<EbsMagnet *>(dev))->read_CorrectorState(att);}
	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
		{return (static_cast<EbsMagnet *>(dev))->is_CorrectorState_allowed(ty);}
};

//	Attribute RequireCycling class definition
class RequireCyclingAttrib: public Tango::Attr
{
public:
	RequireCyclingAttrib():Attr("RequireCycling",
			Tango::DEV_BOOLEAN, Tango::READ) {};
	~RequireCyclingAttrib() {};
	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
		{(static_cast<EbsMagnet *>(dev))->read_RequireCycling(att);}
	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
		{return (static_cast<EbsMagnet *>(dev))->is_RequireCycling_allowed(ty);}
};

//	Attribute Currents class definition
class CurrentsAttrib: public Tango::SpectrumAttr
{
public:
	CurrentsAttrib():SpectrumAttr("Currents",
			Tango::DEV_DOUBLE, Tango::READ_WRITE, 5) {};
	~CurrentsAttrib() {};
	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
		{(static_cast<EbsMagnet *>(dev))->read_Currents(att);}
	virtual void write(Tango::DeviceImpl *dev,Tango::WAttribute &att)
		{(static_cast<EbsMagnet *>(dev))->write_Currents(att);}
	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
		{return (static_cast<EbsMagnet *>(dev))->is_Currents_allowed(ty);}
};

//	Attribute Strengths class definition
class StrengthsAttrib: public Tango::SpectrumAttr
{
public:
	StrengthsAttrib():SpectrumAttr("Strengths",
			Tango::DEV_DOUBLE, Tango::READ_WRITE, 5) {};
	~StrengthsAttrib() {};
	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
		{(static_cast<EbsMagnet *>(dev))->read_Strengths(att);}
	virtual void write(Tango::DeviceImpl *dev,Tango::WAttribute &att)
		{(static_cast<EbsMagnet *>(dev))->write_Strengths(att);}
	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
		{return (static_cast<EbsMagnet *>(dev))->is_Strengths_allowed(ty);}
};


//=========================================
//	Define classes for commands
//=========================================
//	Command Reset class definition
class ResetClass : public Tango::Command
{
public:
	ResetClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	ResetClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~ResetClass() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<EbsMagnet *>(dev))->is_Reset_allowed(any);}
};

//	Command SetDesignStrength class definition
class SetDesignStrengthClass : public Tango::Command
{
public:
	SetDesignStrengthClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	SetDesignStrengthClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~SetDesignStrengthClass() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<EbsMagnet *>(dev))->is_SetDesignStrength_allowed(any);}
};

//	Command ApplyDeltaStrength class definition
class ApplyDeltaStrengthClass : public Tango::Command
{
public:
	ApplyDeltaStrengthClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	ApplyDeltaStrengthClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~ApplyDeltaStrengthClass() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<EbsMagnet *>(dev))->is_ApplyDeltaStrength_allowed(any);}
};

//	Command ResetResonanceStrength class definition
class ResetResonanceStrengthClass : public Tango::Command
{
public:
	ResetResonanceStrengthClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	ResetResonanceStrengthClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~ResetResonanceStrengthClass() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<EbsMagnet *>(dev))->is_ResetResonanceStrength_allowed(any);}
};

//	Command MainCurrentSetpointCheck class definition
class MainCurrentSetpointCheckClass : public Tango::Command
{
public:
	MainCurrentSetpointCheckClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	MainCurrentSetpointCheckClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~MainCurrentSetpointCheckClass() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<EbsMagnet *>(dev))->is_MainCurrentSetpointCheck_allowed(any);}
};

//	Command On class definition
class OnClass : public Tango::Command
{
public:
	OnClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	OnClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~OnClass() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<EbsMagnet *>(dev))->is_On_allowed(any);}
};

//	Command Off class definition
class OffClass : public Tango::Command
{
public:
	OffClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	OffClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~OffClass() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<EbsMagnet *>(dev))->is_Off_allowed(any);}
};

//	Command GetACPSCurrent class definition
class GetACPSCurrentClass : public Tango::Command
{
public:
	GetACPSCurrentClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	GetACPSCurrentClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~GetACPSCurrentClass() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<EbsMagnet *>(dev))->is_GetACPSCurrent_allowed(any);}
};

//	Command GetPSCurrent class definition
class GetPSCurrentClass : public Tango::Command
{
public:
	GetPSCurrentClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	GetPSCurrentClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~GetPSCurrentClass() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<EbsMagnet *>(dev))->is_GetPSCurrent_allowed(any);}
};

//	Command WritePSCurrent class definition
class WritePSCurrentClass : public Tango::Command
{
public:
	WritePSCurrentClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	WritePSCurrentClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~WritePSCurrentClass() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<EbsMagnet *>(dev))->is_WritePSCurrent_allowed(any);}
};

//	Command Cycle class definition
class CycleClass : public Tango::Command
{
public:
	CycleClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	CycleClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~CycleClass() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<EbsMagnet *>(dev))->is_Cycle_allowed(any);}
};

//	Command Swap class definition
class SwapClass : public Tango::Command
{
public:
	SwapClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	SwapClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~SwapClass() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<EbsMagnet *>(dev))->is_Swap_allowed(any);}
};

//	Command Unswap class definition
class UnswapClass : public Tango::Command
{
public:
	UnswapClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	UnswapClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~UnswapClass() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<EbsMagnet *>(dev))->is_Unswap_allowed(any);}
};


/**
 *	The EbsMagnetClass singleton definition
 */

#ifdef _TG_WINDOWS_
class __declspec(dllexport)  EbsMagnetClass : public Tango::DeviceClass
#else
class EbsMagnetClass : public Tango::DeviceClass
#endif
{
	/*----- PROTECTED REGION ID(EbsMagnetClass::Additionnal DServer data members) ENABLED START -----*/
	
	
	/*----- PROTECTED REGION END -----*/	//	EbsMagnetClass::Additionnal DServer data members

	public:
		//	write class properties data members
		Tango::DbData	cl_prop;
		Tango::DbData	cl_def_prop;
		Tango::DbData	dev_def_prop;
	
		//	Method prototypes
		static EbsMagnetClass *init(const char *);
		static EbsMagnetClass *instance();
		~EbsMagnetClass();
		Tango::DbDatum	get_class_property(string &);
		Tango::DbDatum	get_default_device_property(string &);
		Tango::DbDatum	get_default_class_property(string &);
	
	protected:
		EbsMagnetClass(string &);
		static EbsMagnetClass *_instance;
		void command_factory();
		void attribute_factory(vector<Tango::Attr *> &);
		void pipe_factory();
		void write_class_property();
		void set_default_property();
		void get_class_property();
		string get_cvstag();
		string get_cvsroot();
	
	private:
		void device_factory(const Tango::DevVarStringArray *);
		void create_static_attribute_list(vector<Tango::Attr *> &);
		void erase_dynamic_attributes(const Tango::DevVarStringArray *,vector<Tango::Attr *> &);
		vector<string>	defaultAttList;
		Tango::Attr *get_attr_object_by_name(vector<Tango::Attr *> &att_list, string attname);
};

}	//	End of namespace

#endif   //	EbsMagnet_H
